% Constructor of the finite element block class.
%    This class implements irrotational stabilization for fluid elements.
%
%
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options: those recognized by feblock
% and
%   stabfact= stabilization factor, 0 = no stabilization.
%
function retobj = feblock_defor_fluid_irrot_stab (varargin)
    class_name='feblock_defor_fluid_irrot_stab';
    if (nargin == 0)
        parent = feblock;
        self.stabfact=0;
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            parent = feblock(varargin{:});
            options =varargin{1};
            stabfact= 0;
            if isfield(options,'stabfact')
                stabfact= options.stabfact;
            end
            self.stabfact=stabfact;
            self = class(self,class_name,parent);
            retobj = self;
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
