% Constructor of the finite element block class to be used with 
% gap elements.
%
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options: those recognized by feblock
%
function retobj = feblock_defor_gap (varargin)
    class_name='feblock_defor_gap';
    if (nargin == 0)
        parent = feblock;
        self.surface_data = [];
        self.get_gap = [];
        self.get_normal = [];
        self.penalty =  1.0;
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options =varargin{1};
            parent = feblock(varargin{:});
            self.surface_data = [];
            self.get_gap = options.get_gap;
            self.get_normal =  options.get_normal;
            self.penalty =  options.penalty;
            if (isfield(options,'surface_data'))
                self.surface_data = options.surface_data;
            end
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
