% Compute the mass matrices of the individual gcells.
% Return an array of them so they may be assembled.
%   Call as:
%     ems = mass(feb, geom, u);
%   geom=geometry field
%   u=displacement field
%
function ems = mass (self, geom, u)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    dim=get(u, 'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock, 'mater');
    rho = get(get(mat,'property'), 'rho');
     % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Me = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Me{i} =zeros(get(geom,'dim')*nfens);
    end
    matstates = self.matstates;
    Xs = gather(geom, (1:get(geom,'nfens')), 'values', 'noreshape'); % reference coordinates
    Nexp = zeros(dim,dim*nfens);
    for i=1:size(conns,1)
        conn =conns(i,:); % connectivity
        X=Xs(conn,:);
        M = zeros(dim*nfens); % element stiffness matrix
        % Loop over all integration points
        for j=1:npts_per_gcell
            J = X' * Nders{j};% We compute the Jacobian matrix
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, X);
            if (Jac<=0),error('Non-positive Jacobian');end
            for l = 1:nfens; Nexp(1:dim,(l-1)*dim+1:(l)*dim)=eye(dim,dim)*Ns{j}(l); end;
            M = M + Nexp'*Nexp * (rho * Jac * w(j));
        end
        Me{i} = M;
    end
    ems  = elematset(struct('mat',{Me}, 'eqnums',{eqnums}));
end