
%
function E = defor_energy(self, geom, u1, u)
    E=0;
    gcells = get(self.feblock,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');   
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    %npts_per_gcell = length(pc); % number of integration point
    npts=get(integration_rule, 'npts');
    % Material
    mat = get(self, 'mater');
    matstates = get(self, 'matstates');
    % Make the inverse map from finite element nodes to geometric cells
    f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
    gmap=get (f2gmap,'gcell_map');
    fenodemap=get (f2gmap,'fenodes');
    % Pre-allocate the element matrices
    i=0;
    for k = 1:length(gmap)
        gl = gmap{k};
        i = i + length(gl);
    end
    evs(1:i) = deal(elevec);
    iem=1;
    % Now loop over all finite element nodes
    for nix=1:length(gmap)
        gl=gmap{nix};
        thisnn=fenodemap{nix};
        if ~isempty(gl) % This node has a element patch in this block
            % establish local numbering of all nodes of the patch @node nix
            lnmap =0*(1:length(gmap));
            for k=1:length(gl)
                kconn = get(gcells(gl(k)), 'conn'); % connectivity
                for j= 1:length(kconn)
                    if (lnmap(kconn(j)) == 0) & (kconn(j) ~= thisnn)
                        lnmap(kconn(j))=kconn(j);
                    end
                end
            end
            patchconn=lnmap(lnmap>0);
            % now store the local numbers
            for k=1:length(patchconn)
                lnmap(patchconn(k)) =k;
            end
            lnmap(thisnn) =length(patchconn)+1;
            c = gather(geom, thisnn, 'values', 'noreshape'); % reference coordinates of central node
            Rm = material_directions(self,gcells(gl(1)),[],c);
            % preallocate the strain-displacement matrix
            Nspatialderavg=zeros((length(patchconn)+1),dim);
			Vpatch =0;
			for k=1:length(gl)
				kconn = get(gcells(gl(k)), 'conn'); % connectivity
        		X = gather(geom, kconn, 'values', 'noreshape'); % reference coordinates
				U1 = gather(u1, kconn, 'values', 'noreshape'); % displacement in step n+1
				x1 = X + U1; % current coordinates
				pci=find(kconn == thisnn);% at which node in the element are we?
                if (pci<=npts)
                    thisnn_pc=pc(pci,:); % parametric coordinates of that node
%                     Nder = bfundpar(gcells(gl(k)), thisnn_pc);
                    Nder = self.Nder{pci};
                    NderX=bfundsp(gcells(gl(k)),Nder,X*Rm);
                    detJ = Jacobian_volume(gcells(gl(k)), thisnn_pc,X);
                    Ve = detJ*w(pci);
                    Vpatch=Vpatch+Ve;
                    Nspatialderavg(lnmap(kconn),:)= Nspatialderavg(lnmap(kconn),:)+ Ve*NderX;
                end
            end
            Nspatialderavg=Nspatialderavg/ Vpatch;
%             if (self.overlay) Vpatch =Vpatch/2; end
            Xnod=gather(geom,[patchconn thisnn],'values','noreshape');
            U1nod=gather(u1,[patchconn thisnn],'values','noreshape');
            x1nod=Xnod+U1nod;
            F1nodal= x1nod'*Nspatialderavg;
%             Bnodal= self.hBlmat (self,Nspatialderavg*inv(F1nodal),c,Rm);
            [dE,matstates{i,j}] = update(mat, matstates{nix,1}, struct ('output','strain_energy','F',F1nodal));
            E = E + dE *(Vpatch *det(F1nodal)); % note the sign
            %[sigma,matstates{gl(k),pci}] = update(mat, matstates{nix,1},...
            %    struct ('output','Cauchy','F',F1nodal));
            %Fe = -(Bnodal'* sigma) * (Vpatch *det(F1nodal)); % note the sign
            %evs(iem) = set(evs(iem), 'vec', Fe);
            %evs(iem) = set(evs(iem), 'eqnums', gather(u1, [patchconn thisnn], 'eqnums'));
            iem=iem+1;
        end
    end
    self.matstates = matstates;
    return;
end
