% Compute the local elastic stiffness matrix. 
%
% function SM = local_stiffness(self, E, G, A, I2, I3, J, L)
% 
% Inputs:
% E, G= Young's and shear modulus, 
% A= cross-sectional area, 
% I2, I3=central moment of inertia of the cross-section about the x2 and x3 
% coordinate axis, 
% J=St Venant torsion constant, 
% L= current length of the element, 
% 
% Outputs:
% SM = local stiffness matrix, 12 x 12
function SM = local_stiffness(self, E, G, A, I2, I3, J, L)
    aN = local_cartesian_to_natural (self, L);
    DN = natural_stiffness(self, E, G, A, I2, I3, J, L);
    SM=aN'*DN*aN;
end

% Below is just a collection of previously useful code.
%     Grbm = rigid_body_matrix(self,L);
%     Db= [EY*CSA/L,0,0,0,0,0;
%         0,G*IT/L,0,0,0,0;
%         0,0,4*EY*IN2/L,2*EY*IN2/L,0,0;
%         0,0,2*EY*IN2/L,4*EY*IN2/L,0,0;
%         0,0,0,0,4*EY*IN3/L,2*EY*IN3/L;
%         0,0,0,0,2*EY*IN3/L,4*EY*IN3/L];
%     SM=Grbm'*Db*Grbm;
   
%     Symbolic derivation possible with the code
%    syms EY CSA L IN2 IN3 TST real
% Grbm = [-1,0,0,0,0,0,1,0,0,0,0,0;
%         0,0,0,-1,0,0,0,0,0,1,0,0;
%         0,0,-1/L,0,1,0,0,0,1/L,0,0,0;
%         0,0,-1/L,0,0,0,0,0,1/L,0,1,0;
%         0,1/L,0,0,0,1,0,-1/L,0,0,0,0;
%         0,1/L,0,0,0,0,0,-1/L,0,0,0,1;
%        ];
%    Db= [EY*CSA/L,0,0,0,0,0;
%        0,TST/L,0,0,0,0;
%        0,0,4*EY*IN2/L,2*EY*IN2/L,0,0;
%        0,0,2*EY*IN2/L,4*EY*IN2/L,0,0;
%        0,0,0,0,4*EY*IN3/L,2*EY*IN3/L;
%        0,0,0,0,2*EY*IN3/L,4*EY*IN3/L];
%    SM=Grbm'*Db*Grbm
%    
%    simple (SM)
