% Compute the mass matrices of the individual gcells.
% Return an array of them so they may be assembled.
%   Call as:
%     ems = mass(feb, geom, u);
%   geom=geometry field
%   u=displacement field
%
function ems = mass (self, geom, u)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:size(pc,1)
        N{j} = bfun(gcells, pc(j,:));
        Nder{j} = bfundpar(gcells, pc(j,:));
    end
    mat = get(self, 'mater');% Material
    rho = get(get(mat,'property'), 'rho');
    Nexp = zeros(dim,dim*nfens);
    conns = get(gcells, 'conn'); % connectivity
    Mes= cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =[];
        Mes{i} = [];
    end
    % Now loop over all gcells in the block
    Me = zeros(dim*nfens); %
    xs =get(geom,'values');
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        Me =Me*0;% zero out element matrix
        % Loop over all integration points
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nder{j},x);
            Jac = Jacobian_volume(gcells,conn, N{j}, J, x);
            for l = 1:nfens;
                Nexp(1:dim,(l-1)*dim+1:(l)*dim)=eye(dim)*N{j}(l);
            end;
            Me = Me + Nexp'*Nexp * rho * Jac * w(j);
        end
        Mes{i} =  Me;
        eqnums{i} = gather(u, conn, 'eqnums');
    end
    ems  = elematset(struct('mat',{Mes}, 'eqnums',{eqnums}));
end