%  Draw graphic representation for material states in all gcells.
%
% function draw_integration_points(self, gv, context)
%
% Input arguments
% self = self
% gv = graphic viewer
% context = struct
% with typically mandatory fields
%    x=reference geometry field
%    u=displacement field to be used for the calculation of the stresses
%    u_displ=displacement field to be used for display
%    dT=temperature difference field
% with optional fields
%    scale = scale the representation of the material state graphic by this
%       factor
%    dT=temperature difference (with respect to the reference temperature);
%       temperature differences assumed to be zero if it is not supplied.
% Any other fields in the context struct may be interpreted by the
% implementation of the material: see the draw() function for the
% particular material.
% 
function draw_integration_points(self, gv, context)
    gcells = get(self.feblock_defor_ss,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(context.x,'dim');
    % Integration rule
    integration_rule = get(self.feblock_defor_ss, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = length(pc); % number of integration point
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock_defor_ss, 'mater');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Xs =gather(context.x,(1:get(context.x,'nfens')),'values','noreshape');
    Blmat =get(self,'Blmat');
    matstates = self.matstates;
    context.update_context= [];
    context.ms = [];
    for i=1:ngcells
        conn =conns(i,:);
        X=Xs(conn,:);
        U_displ = gather(context.u_displ, conn, 'values', 'noreshape'); % displacement
        U = gather(context.u, conn, 'values'); % displacement
        if isfield(context,'dT')
            dTs = gather(context.dT, conn, 'values', 'noreshape'); % displacement
        else
            dTs = zeros(nfens,1);
        end
        for j=1:npts_per_gcell
            c =X'*self.N{j};% physical location
            J = X' * self.Nder{j};%J = Jacobian_matrix(gcells,Nders{j},x);%J = x' * Nder;
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            Jac = Jacobian_volume(gcells,conn, self.N{j}, J, X);
            if (isempty(Rm))
                Ndersp = self.Nder{j}/J;% derivatives wrt local coor
            else
                Ndersp = self.Nder{j}/(Rm'*J);% derivatives wrt local coor
            end
            B = Blmat (self, self.N{j},Ndersp, c, Rm);
            context.update_context.strain = B*U;
            context.update_context.dT = dTs'*self.N{j};
            context.ms = matstates{i,j};
            context.xyz =self.N{j}'*(X+U_displ);
            draw(mat, gv, context);
        end
    end
end
