% stiffness_geo: Compute the geometric stiffness matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness_geo(feb, geom, u1, u)
%     geom=reference geometry field
%     u1=displacement field in the *current* configuration
%     u=displacement field in the last known configuration
%    Returns an array of elemat objects that may be assembled
%    into the system stiffness matrix.
%
function ems = stiffness_geo (self, geom, u1,u)
    gcells = get(self.feblock_defor_ss,'gcells');
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    idx = (1:dim:(nfens-1)*dim+1);
    % Integration rule
    integration_rule = get(self.feblock_defor_ss, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock_defor_ss, 'mater');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Ke{i} =zeros(get(geom,'dim')*nfens);
    end
    Xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    Blmat =get(self,'Blmat');
    matstates = self.matstates;
    K1 = zeros(dim*nfens);    % auxiliary matrix
    context.dT=[];
    for i=1:size(conns,1)
        conn =conns(i,:);
        X=Xs(conn,:);
        U1 = gather(u1, conn, 'values');  % displacements of nodes
        for j=1:npts_per_gcell
            c =X'*self.N{j};% physical location
            J = X' * self.Nder{j};%J = Jacobian_matrix(gcells,Nders{j},x);%J = x' * Nder;
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            Jac = Jacobian_volume(gcells,conn, self.N{j}, J, X);
            if (isempty(Rm))
                Ndersp = self.Nder{j}/J;% derivatives wrt local coor
            else
                Ndersp = self.Nder{j}/(Rm'*J);% derivatives wrt local coor
            end
            B = Blmat (self, self.N{j},Ndersp, c, Rm);
            context.strain =  B*U1;
            [cauchy,matstates{i,j}] = update(mat, matstates{i,j}, context);% cauchy is cauchy's stress tensor
            sigma = stress_6v_to_3x3t(mat,cauchy);
            c1 = Ndersp*sigma*Ndersp';
            K1(idx,idx)     = c1;
            K1(idx+1,idx+1) = c1;
            K1(idx+2,idx+2) = c1;
            Ke{i} = Ke{i} + K1 *(Jac*w(j));
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
