% Constructor of the finite element block class to be used with small
% displacement, small strain    deformation mechanics for continuum,
% isoparametric elements. Implements methods for velocity-proportional
% damping.
%
% Note: Selective reduced integration is used; see Hughes (1987).
%
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options: those recognized by feblock
%
function retobj = feblock_defor_ss_sri_damped (varargin)
    class_name='feblock_defor_ss_sri_damped';
    if (nargin == 0)
        parent = feblock_defor_ss_sri;
        self.damping_abc=0;
        self.N= [];
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options =varargin{1};
            parent = feblock_defor_ss_sri(varargin{:});
            self.damping_abc=0;
            self.N= [];
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end