% Compute the secant nodal capacity matrices of the individual gcells. 
%
% function ems = lumped_volume(self, geom, temp)
%
% Return an array of them so they may be assembled.
%  
%    where
%       geom=geometry field
%       temp = field used to retrieve equation numbers   
% 
function ems = secant_capacity(self, geom, temp)
    gcells = get(self.feblock_diffusion,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    % Pre-allocate the element matrices
    ems(1:ngcells) = deal(elemat);
    % Integration rule
    integration_rule = get(self.feblock_diffusion, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Material
    mat = get(self.feblock_diffusion, 'mater');
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = get(gcells(i), 'conn'); % connectivity
        x = gather(geom, conn, 'values', 'noreshape'); % coord
        T = gather(temp,conn,'values');% node temperatures
        Ce = zeros(nfens); % element matrix
        for j=1:npts_per_gcell % Loop over integration points
            N = self.N{j};
            detJ = Jacobian_volume(gcells(i),pc(j,:),x);
            Ce = Ce + N*N' * (detJ * w(j));
        end
        ems(i)=set(ems(i),'mat', Ce);
        ems(i)=set(ems(i),'eqnums',gather(temp,conn,'eqnums'));
    end
    return;
end
