% Constructor of the map from finite element nodes to the geometric cells.
%
% function retobj = fenode_to_gcell_map (varargin)
%
% Constructor of the map from finite element nodes to the geometric cells
% that connect the nodes.
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>. 
% Parameters:
%    gcells =array  of geometric cells
%
function retobj = fenode_to_gcell_map (varargin)
class_name='fenode_to_gcell_map';
parent=classbase;
if (nargin == 0)
    self.gcell_map = {};
    self = class(self, class_name, parent);
    retobj = self;
    return;
elseif (nargin == 1)
    arg = varargin{1};
    if strcmp(class(arg),class_name) % copy constructor.
        retobj = arg;
        return;
    else
        Parameters = varargin{1};
        self.gcell_map = {};
        conns=get(Parameters.gcells,'conn');
        for i=1:count(Parameters.gcells)
            conn=unique(conns(i,:));
            for j=1:length(conn)
                self.gcell_map{conn(j)}=[];
            end
        end
        for i=1:count(Parameters.gcells)
            conn=unique(conns(i,:));
            for j=1:length(conn)
                m=self.gcell_map{conn(j)};
                m=[m i];
                self.gcell_map{conn(j)}=m;
            end
        end
        self = class(self, class_name, parent);
        retobj = self;
        return;
    end
else
    error('Illegal arguments!');
    return;
end
return;

