% Evaluate the basis function matrix for an 27-node brick.
%
% function val = bfun(self,param_coords)
%
%   Call as:
%      N = bfun (g, pc)
%   where
%      g=gcell,
%      pc=parametric coordinates, -1 < pc(j) < 1, length(pc)=3.
%
function N = bfun(self,param_coords)
    xi=param_coords(1);
    et=param_coords(2);
    ze=param_coords(3);
    
    A =1/4096; B =9/4096; C=81/4096; D =729/4096;
    e1 =(9*xi^2-1); e2 =(9*et^2-1); e3 =(9*ze^2-1);
    e4 =(xi^2-1); e5 =(et^2-1); e6 =(ze^2-1);
    e7 =(3*xi-1); e8 = (3*et-1); e9 =(3*ze-1);
    e10 =(3*xi+1); e11 = (3*et+1); e12 =(3*ze+1);
    e13=(xi+1); e14 =(xi-1); e15 =(et+1); e16 =(et-1); e17 =(ze+1); e18 =(ze-1);
    N=[-A*e18*e14*e16*e3*e1*e2;
        B*e18*e7*e16*e3*e4*e2;
        -B*e18*e10*e16*e3*e4*e2;
        A*e18*e13*e16*e3*e1*e2;
        B*e18*e14*e8*e3*e1*e5;
        -C*e18*e7*e8*e3*e4*e5;
        C*e18*e10*e8*e3*e4*e5;
        -B*e18*e13*e8*e3*e1*e5;
        -B*e18*e14*e11*e3*e1*e5;
        C*e18*e7*e11*e3*e4*e5;
        -C*e18*e10*e11*e3*e4*e5;
        B*e18*e13*e11*e3*e1*e5;
        A*e18*e14*e15*e3*e1*e2;
        -B*e18*e7*e15*e3*e4*e2;
        B*e18*e10*e15*e3*e4*e2;
        -A*e18*e13*e15*e3*e1*e2;
        B*e9*e14*e16*e6*e1*e2;
        -C*e9*e7*e16*e6*e4*e2;
        C*e9*e10*e16*e6*e4*e2;
        -B*e9*e13*e16*e6*e1*e2;
        -C*e9*e14*e8*e6*e1*e5;
        D*e9*e7*e8*e6*e4*e5;
        -D*e9*e10*e8*e6*e4*e5;
        C*e9*e13*e8*e6*e1*e5;
        C*e9*e14*e11*e6*e1*e5;
        -D*e9*e7*e11*e6*e4*e5;
        D*e9*e10*e11*e6*e4*e5;
        -C*e9*e13*e11*e6*e1*e5;
        -B*e9*e14*e15*e6*e1*e2;
        C*e9*e7*e15*e6*e4*e2;
        -C*e9*e10*e15*e6*e4*e2;
        B*e9*e13*e15*e6*e1*e2;
        -B*e12*e14*e16*e6*e1*e2;
        C*e12*e7*e16*e6*e4*e2;
        -C*e12*e10*e16*e6*e4*e2;
        B*e12*e13*e16*e6*e1*e2;
        C*e12*e14*e8*e6*e1*e5;
        -D*e12*e7*e8*e6*e4*e5;
        D*e12*e10*e8*e6*e4*e5;
        -C*e12*e13*e8*e6*e1*e5;
        -C*e12*e14*e11*e6*e1*e5;
        D*e12*e7*e11*e6*e4*e5;
        -D*e12*e10*e11*e6*e4*e5;
        C*e12*e13*e11*e6*e1*e5;
        B*e12*e14*e15*e6*e1*e2;
        -C*e12*e7*e15*e6*e4*e2;
        C*e12*e10*e15*e6*e4*e2;
        -B*e12*e13*e15*e6*e1*e2;
        A*e17*e14*e16*e3*e1*e2;
        -B*e17*e7*e16*e3*e4*e2;
        B*e17*e10*e16*e3*e4*e2;
        -A*e17*e13*e16*e3*e1*e2;
        -B*e17*e14*e8*e3*e1*e5;
        C*e17*e7*e8*e3*e4*e5;
        -C*e17*e10*e8*e3*e4*e5;
        B*e17*e13*e8*e3*e1*e5;
        B*e17*e14*e11*e3*e1*e5;
        -C*e17*e7*e11*e3*e4*e5;
        C*e17*e10*e11*e3*e4*e5;
        -B*e17*e13*e11*e3*e1*e5;
        -A*e17*e14*e15*e3*e1*e2;
        B*e17*e7*e15*e3*e4*e2;
        -B*e17*e10*e15*e3*e4*e2;
        A*e17*e13*e15*e3*e1*e2];

%     % Symbolic derivation of the matrix of basis function values
%     x=sym('x','real');
%     xi=sym('xi','real');
%     et=sym('et','real');
%     ze=sym('ze','real');
%     a=1/3;
%     x0 =-1;
%     l1 =eval(((x-1)*(x-a)*(x-(-a)))/((x0-1)*(x0-a)*(x0-(-a))));
%     x0 =-a;
%     l2 =eval(((x-1)*(x-a)*(x-(-1)))/((x0-1)*(x0-a)*(x0-(-1))));
%     x0 =+a;
%     l3 =eval(((x-1)*(x-(-a))*(x-(-1)))/((x0-1)*(x0-(-a))*(x0-(-1))));
%     x0 =+1;
%     l4 =eval(((x-a)*(x-(-a))*(x-(-1)))/((x0-a)*(x0-(-a))*(x0-(-1))));
%     subs(l1,'x','et')
%     xis = [subs(l1,'x','xi'),subs(l2,'x','xi'),subs(l3,'x','xi'),subs(l4,'x','xi')];
%     ets = [subs(l1,'x','et'),subs(l2,'x','et'),subs(l3,'x','et'),subs(l4,'x','et')];
%     zes = [subs(l1,'x','ze'),subs(l2,'x','ze'),subs(l3,'x','ze'),subs(l4,'x','ze')];
%     xisets=(xis'*ets);
%     N(:,:,1)=xisets*zes(1);
%     N(:,:,2)=xisets*zes(2);
%     N(:,:,3)=xisets*zes(3);
%     N(:,:,4)=xisets*zes(4);
%     for ze=[-1,-a,+a,+1]
%         for et=[-1,-a,+a,+1]
%             for xi=[-1,-a,+a,+1]
%                 n=double(subs(subs(subs(N,'ze',num2str(ze)),'et',num2str(et)),'xi',num2str(xi)));
%                 i=find(n>0.99);
%                 disp([ char(simple(N(i))) ';'])
%             end
%         end
%     end
% 
%     x=sym('x','real');
%     xi=sym('xi','real');
%     et=sym('et','real');
%     ze=sym('ze','real');
%     a=1/3;
%     x0 =-1;
%     l1 =eval(((x-1)*(x-a)*(x-(-a)))/((x0-1)*(x0-a)*(x0-(-a))));
%     x0 =-a;
%     l2 =eval(((x-1)*(x-a)*(x-(-1)))/((x0-1)*(x0-a)*(x0-(-1))));
%     x0 =+a;
%     l3 =eval(((x-1)*(x-(-a))*(x-(-1)))/((x0-1)*(x0-(-a))*(x0-(-1))));
%     x0 =+1;
%     l4 =eval(((x-a)*(x-(-a))*(x-(-1)))/((x0-a)*(x0-(-a))*(x0-(-1))));
%     subs(l1,'x','et')
%     xis = [subs(l1,'x','xi'),subs(l2,'x','xi'),subs(l3,'x','xi'),subs(l4,'x','xi')];
%     ets = [subs(l1,'x','et'),subs(l2,'x','et'),subs(l3,'x','et'),subs(l4,'x','et')];
%     zes = [subs(l1,'x','ze'),subs(l2,'x','ze'),subs(l3,'x','ze'),subs(l4,'x','ze')];
%     xisets=(xis'*ets);
%     N(:,:,1)=xisets*zes(1);
%     N(:,:,2)=xisets*zes(2);
%     N(:,:,3)=xisets*zes(3);
%     N(:,:,4)=xisets*zes(4);
%     for ze=[-1,-a,+a,+1]
%         for et=[-1,-a,+a,+1]
%             for xi=[-1,-a,+a,+1]
%                 n=double(subs(subs(subs(N,'ze',num2str(ze)),'et',num2str(et)),'xi',num2str(xi)));
%                 i=find(n>0.99);
%                 disp(['N(' num2str(i) ',:)=[' char(simple(diff(N(i),'xi'))) ',' char(simple(diff(N(i),'et'))) ',' char(simple(diff(N(i),'ze'))) '];'])
%             end
%         end
%     end

end