% Constructor of simplex nodal-quadrature rule.
% 
% function retobj = simplex_nq_rule (varargin)
% 
%   Parameters=structure with fields
%      dim=number of  space dimensions
%            3=3-D rule (four-node tetrahedron), 2=2-D  rule (three-node triangle).
%      order=linear (1) or quadratic (2)
% 
%  Example:
%      ir = simplex_nq_rule(dim);
% or
%      ir = simplex_nq_rule(Parameters);
% 
function retobj = simplex_nq_rule (varargin)
    class_name='simplex_nq_rule';
    parent=classbase;
    if (nargin == 0)
        self.alpha= 0;
        self.dim = 0;
        self.order =1;
        self.param_coords = [];
        self.weights= [];
        retobj = class(self,class_name,parent);
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            overlay = false;
            self.alpha= 0.6/6;
            if  ~(strcmp(class(varargin{1}),'double'))
                Parameters =varargin{1};
                self.dim = Parameters.dim;
                if (isfield(Parameters,'order'))
                    self.order =Parameters.order;
                end
                if (isfield(Parameters,'alpha'))
                    self.alpha =Parameters.alpha;
                end
            else
                self.dim = varargin{1};
                self.order =1;
            end
            if (self.order== 1)
                switch self.dim
                    case 1
                        self.param_coords = [-1;+1];
                        self.weights = [1,1];
                    case 2
                        self.param_coords = [ 0, 0; 1, 0; 0, 1];
                        self.weights = [1/3 1/3 1/3]/2;
                    case 3
                        self.param_coords = [0, 0, 0; 1, 0, 0; 0, 1, 0; 0, 0, 1];
                        self.weights = [1, 1, 1, 1]/6/ 4;
                    otherwise
                        error('Unsupported Dimension');
                end
            elseif (self.order== 2)
                switch self.dim
                    case 2
                        self.param_coords = [ 0, 0; 1, 0; 0, 1; 1/2, 0; 1/2,1/2; 0, 1/2];
                        self.weights = [0 0 0 1/3 1/3 1/3]/2;
%                         self.param_coords = [ 0, 0; 1, 0; 0, 1; 1/2, 0; 1/4,1/4; 0, 1/2];
%                         self.weights = ones(1,6)/6/2;
                    case 3
                        self.param_coords = [0,0,0;1,0,0;0,1,0;0,0,1;0.5,0,0;0.5,0.5,0;0,0.5,0;0,0,0.5;0.5,0,0.5;0,0.5,0.5];
                        b=(1- 6*self.alpha)/4;
                        self.weights = [[1, 1, 1, 1]*b,[1, 1, 1, 1, 1, 1]*self.alpha]/6;
                    otherwise
                        error('Unsupported Dimension');
                end
            elseif (self.order== 3)
                switch self.dim
                    case 2
                        self.param_coords = [ 0, 0; 1, 0; 0, 1; 1/2, 0; 1/2, 1/2; 0, 1/2; 1/3, 1/3];
                        self.weights = [1/40 1/40 1/40 1/15 1/15 1/15 9/40];
%                         self.param_coords = [ 0, 0; 1, 0; 0, 1; 1/2, 0; 1/4,1/4; 0, 1/2];
%                         self.weights = ones(1,6)/6/2;
                    otherwise
                        error('Unsupported Dimension');
                end
            else
                error('Unsupported  order');
            end
            if (overlay)
                self.weights =self.weights/2;
            end
            retobj = class(self,class_name,parent);
        end
    else
        error('Illegal arguments!');
    end
    return;
end

