% Advance the state by specified number of steps
%   Call as:
%      advance(obj, torque, dt, nsteps)
%   where
%      obj       - object
%      torque    - handle to function defining spatial torque
%      dt        - time step
%      nsteps    - number of steps to take
%
%   The first form returns the list of property names as a string,
%   the second form returns the value of the named property.
%
function retobj = advance(self,torque,dt,nsteps)
    tol=100*eps;
    Omegan1 = self.Omega;
    Rn1 = self.R;
    t=get(self.rotint,'t');
    I=self.I;
    % calculate effective loads at t_n-1
    tn1 = feval(torque, t, Rn1);
    for i=1:nsteps
        Omegan=Omegan1;
        i=0; maxi=150;
        pTheta=0*Omegan1;
        while (1)
            Theta=dt/2*(Omegan+Omegan1);
            DRn=rotmat(Theta);
            Rn=reortho(Rn1*DRn);
            tn=feval(torque, t+dt, Rn);
            Omegan=I\(Rn'*(Rn1*I*Omegan1+dt/2*(tn+tn1)));
            i=i+1;
            if (i > maxi)
                if (max(abs(Theta-pTheta)) > 1e-9)
                    warning(['bb_rkmk_trap - Failed to converge: ||residual||=' num2str(norm(Theta-pTheta))]);
                end
                break;
            end
            if (norm(Theta-pTheta) < tol)
                break;
            end
            pTheta=Theta;
        end
        % swap variables for next step
        Rn1=Rn;
        Omegan1=Omegan;
        tn1 =tn;
        t=t+dt;
    end
    self.R = Rn;
    self.Omega = Omegan1;
    self.rotint = set(self.rotint,'t',t);
    retobj=self;
end
