% Baffled circular piston, prescribed acceleration
rho=1.21e-3*1e-9;% mass density of air
c =343*1000;% sound speed of air, millimeters per second
bulk= c^2*rho;
R=50;% length of the box, millimeters
Ro=500; % width of the box, millimeters
a_piston=-1e-3;% that is into the fluid
tol=R/10000;
nref=2;%
nlayers=28;
graphics= true;% plot the solution as it is computed?

[fens,gcells]=H8_sphere(R,nref);


bg=mesh_bdry(gcells);
l=gcell_select(fens,bg,struct ('facing', true, 'direction', [1,1,1]));
% gv=drawmesh({fens,gcells1},'gcells','facecolor','white')
[fens1,gcells1] = Q4_extrude(fens,subset(bg,l),nlayers,@(xyz, layer)(R+layer/nlayers*(Ro-R))*xyz/norm(xyz));
%     gcells1=set(gcells1,'label',2);
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
gcells=cat(gcells1,gcells2);

options. distance=R;
options. inflate=tol;
l1=gcell_select(fens, bg, options);
options. facing=true;
options. direction=[-1,0,0];
l2=gcell_select(fens, bg, options);
piston_gcells=subset(bg,intersect(l1,l2));


gv=drawmesh({fens,mesh_bdry(gcells)},'gcells','facecolor','white','facealpha',0.2);
gv=drawmesh({fens,piston_gcells},'gv',gv,'gcells','facecolor','yellow');

boundary_gcells=mesh_bdry(gcells);
outerl=gcell_select(fens,boundary_gcells,struct ('facing', true, 'direction', [1,1,1]));

prop=property_viscous_fluid(struct('bulk',bulk,'rho',rho));
mater=mater_defor_ss_linviscel_triax (struct('property',prop));
feb = feblock_acoustics (struct (...
    'mater',mater,'gcells',gcells,...
    'integration_rule',gauss_rule(3,2)));
pfeb = feblock_acoustics (struct (...
    'mater',mater,'gcells',piston_gcells,...
    'integration_rule',gauss_rule(2,2)));

geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
P = field(struct ('name',['P'], 'dim', 1,'nfens',get(geom,'nfens')));
ebc_fenids=connected_nodes (subset(boundary_gcells,outerl));
    ebc_prescribed=ones(1,length (ebc_fenids));
    ebc_comp=[];
    ebc_val=ebc_fenids*0;
    P = set_ebc(P, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    P = apply_ebc (P);
P = numbereqns (P);

% S = start (sparse_sysmat, get(P, 'neqns'));
% S = assemble (S, acoustic_stiffness(feb, geom, P));
C = start (sparse_sysmat, get(P, 'neqns'));
C = assemble (C, acoustic_mass(feb, geom, P));

fi= force_intensity(struct('magn',-rho*a_piston));
F = start (sysvec, get(P, 'neqns'));
F = assemble (F, pressure_flux_loads(pfeb, geom, P, fi));
P = scatter_sysvec(P, C\F);



if graphics
    Camera =1.0e+003 * [-1.3364   -3.0752   -2.0523    0.2492    0.2492    0.2493         0         0    0.0010    0.0084];
    xyz=get(geom,'values');
    box =reshape([min(xyz);max(xyz)],1,6);
    bfeb = feblock(struct ('gcells',boundary_gcells));
    gv=graphic_viewer;
    gv=reset (gv,[]);
    scale=0.05;
    wv = get(P,'values');
    dcm=data_colormap(struct ('range',[min(wv),max(wv)], 'colormap',nastrancolors));
    colors=map_data(dcm, wv);
    colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
    gv=reset (gv,[]);
    camset(gv,Camera);
    draw(bfeb,gv, struct ('x', geom,'u',0*geom,'colorfield',colorfield));
    draw_axes (gv,struct('length',20 ));
    pause(0.55);
end
