%%  Cantilever quarter-circular arch, buckling

%%
function [dts,Tip_uys]=ArchTransientConv8
    %%
    % Parameters:
    % The parameters correspond to aluminum.
    E=71240.0;%MPa
    nu=0.31;% Poisson ratio
    rho=2.7e-9;
    b=0.6; h=30; L=240;% cross-sectional dimensions and length of each leg in millimeters
    vmagn=-1e+2;% Magnitude of the total applied force, Newton
    tend= 0.5;
    color=hot(13);
    Newmarkg = 1/2; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    Report =@(x) (0);%@(x) (disp(x) );
    graphics= ~true;
    nrefs=13;
    dts=[];
    Tip_uys=[];
    for nref=nrefs
        nsteps=(10*2^nref);
        dt=tend/nsteps;
        dts=[dts,dt];
        %%
        % Mesh
        n=8*10;
        tolerance=L/n/100;
        Parameters=struct ('A',b*h,'I2',b*h^3/12,'I3',b^3*h/12,'J',b^3*h/3,...
            'x1x2_vector',[0 1 0], 'label',1);
        % The coordinates of the points are on circle.
        X=L*[cos(pi/2*(0:1:(n-1))/(n-1))',zeros(n,1),sin(pi/2*(0:1:(n-1))/(n-1))'];
        [fens,gcells] = Beam_member(X,n,@gcellset_beam3,Parameters);
        
        % The clamped end is
        clampedn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));
        % The force is applied at the tip
        tipn=fenode_select(fens, struct('box', update_box([],[L,0,0]), 'inflate', tolerance));
        
        %%
        % Properties
        prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
        mater = mater_defor_ss_linel_triax (struct('property',prop));
        
        %%
        % Finite element block. Linear beam element formulation is used.
        feb = feblock_defor_ss_beam3 (struct ('mater',mater, 'gcells',gcells));
        
        %%
        % Geometry and displacement fields
        geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
        ur0  = field(struct ('name',['ur0'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
        
        
        %%
        % Apply EBC's
        ebc_fenids=[clampedn];
        ebc_prescribed=[1];
        ebc_comp=[];
        ebc_val=ebc_fenids*0;
        ur0  = set_ebc(ur0, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        ur0  = apply_ebc (ur0);
        
        
        %%
        % Number equations
        ur0  = numbereqns (ur0);
        neqns=get(ur0, 'neqns');
        
        
        %%
        % Assemble the System matrices
        K = start (sparse_sysmat, neqns);
        K = assemble (K, stiffness(feb, geom, ur0));
        M = start (sparse_sysmat, neqns);
        M = assemble (M, mass(feb, geom, ur0));
        Km=get(K, 'mat');
        Mm=get(M, 'mat');
        Am=(Km+(1/(Newmarkb*dt^2))*Mm);
        
        % Initial conditions
        ur0 = ur0*0; % zero out the displacement
        v0 = ur0*0; % zero out the Velocity
        V0Values = gather(v0,(1:count(fens)),'values','noreshape');
        xyz = gather(geom,(1:count(fens)),'values','noreshape');
        for j=1:size(xyz,1 )
            if (xyz(j,3)<L/10)
                V0Values(j,2)= vmagn*(L/10-xyz(j,3))/(L/10);
            end
        end
        v0 = scatter(v0,(1:count(fens)),V0Values);
        a0 = ur0*0; % zero out the Acceleration
        
        if (graphics )
            gv=graphic_viewer;
            gv=reset (gv,[]);
            scale=50;
        end
        
        tipu=gather(ur0,tipn, 'values', 'noreshape');
        Tip_uy=[tipu(2)];
        t=0; %
        for step=1:nsteps
            t = t + dt;
            Report(['Time ' num2str(t) ]); % pause
            % Initialization
            ur0 = apply_ebc(ur0);% Apply boundary conditions
            a1 = -(1/Newmarkb/dt)*v0 -(1/2-Newmarkb)/Newmarkb*a0;% Predictor
            v1 = v0 + dt*((1-Newmarkg)*a0 + Newmarkg*a1);% Predictor
            
            rhs=-Km*gather_sysvec(ur0)+Mm*((1/(Newmarkb*dt^2))*gather_sysvec(dt*v0 +(dt^2/2*(1-2*Newmarkb))*a0));
            dur=scatter_sysvec(ur0, Am\rhs);
            ur1 = ur0 + dur;   % increment displacement
            v1 = v1 + (Newmarkg/Newmarkb/dt)*dur;
            a1 = a1 + (1/Newmarkb/dt^2)*dur;
            
            ur0 = ur1;       % update the displacement
            v0 = v1;       % update the velocities
            a0 = a1;       % update the accelerations
            
            if graphics
                gv=reset(gv,[]);
                transl=slice(ur1,(1:3),'transl');
                rotat=slice(ur1,(4:6),'rotat');
                draw(feb, gv, struct ('x', geom, 'u', 0*transl, 'rot', 0*rotat, 'facecolor','none'));
                draw(feb, gv, struct ('x', geom, 'u', scale*transl, 'rot', scale*rotat));
                %         draw_local_coordinates(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'length',12.5));
                %             camset(gv, camera);
                drawnow;
            end
            tipu=gather(ur1,tipn, 'values', 'noreshape');
            Tip_uy=[Tip_uy;tipu(2)];
        end
%         plot(linspace(0,tend,length(Tip_uy)),Tip_uy,'color',color(nref,:),'linewidth',2); hold on
%         xlabel('Time');
%         ylabel('Tip transverse displacement');
%         set_graphics_defaults
%         set(gca,'xlimmode','manual');
%         %                 set(gca,'ylimmode','manual');
%         %                 set(gca,'DataAspectRatioMode','auto');
%         set(gca,'xlim', [0,tend]);
%         %                 set(gca,'ylim',
%         %                 1.2*[-10,+10]);
%         grid on
%         drawnow;figure(gcf);
        t,Tip_uys=[Tip_uys,Tip_uy(end)]
    end
end
function report_convergence
dts=[
    2.500000000000000e-002
    1.250000000000000e-002
    6.250000000000000e-003
    3.125000000000000e-003
    1.562500000000000e-003
    7.812500000000000e-004
    3.906250000000000e-004
    1.953125000000000e-004
    9.765625000000001e-005
    4.882812500000000e-005
    2.441406250000000e-005
    1.220703125000000e-005
    6.103515625000000e-006];
Tip_uys=[ 3.122085648411475e-001
    3.418983788327313e-001
    3.845443649165293e-001
    2.067909971933999e-001
    2.222042317510485e-001
    1.611813045152370e-001
    1.745681197134975e-001
    1.705301988214444e-001
    1.842558410283465e-001
    1.907328018714345e-001
    1.937062270658944e-001
    1.927787286758897e-001
    1.929594924049057e-001];
loglog(dts(2:end), abs(diff(Tip_uys)),'o-');
grid on
end

function report_h_convergence
hs=[4; 2; 1; 0.5];
Tip_uys=[2.697851306258768e-001
    2.090212325861665e-001
    2.012081249292549e-001
    1.927787286758897e-001
    ];
loglog(hs(2:end), abs(diff(Tip_uys)),'o-');
grid on
end