%%  Cantilever quarter-circular arch, buckling

%%
function CBArchTransient
    %%
    % Parameters:
    % The parameters correspond to aluminum.
    E=71240.0;%MPa
    nu=0.31;% Poisson ratio
    rho=2.7e-9;
    b=0.6; h=30; L=240;% cross-sectional dimensions and length of each leg in millimeters
    vmagn=-1e+2;% Magnitude of the total applied force, Newton
    tend= 0.5;
    dt=tend/200;
    Newmarkg = 1/2; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    maxit=30;
    Report =@(x) (disp(x) );
    graphics= ~true;
    
    %%
    % Mesh
    n=30;
    tolerance=L/n/100;
    Parameters=struct ('A',b*h,'I2',b*h^3/12,'I3',b^3*h/12,'J',b^3*h/3,...
        'x1x2_vector',[0 1 0], 'label',1);
    % The coordinates of the points are on circle.
    X=L*[cos(pi/2*(0:1:(n-1))/(n-1))',zeros(n,1),sin(pi/2*(0:1:(n-1))/(n-1))'];
    [fens,gcells] = Beam_member(X,n,@gcellset_beam3,Parameters);
    
    % The clamped end is
    clampedn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));
    % The force is applied at the tip
    tipn=fenode_select(fens, struct('box', update_box([],[L,0,0]), 'inflate', tolerance));
    
    
    % Choose the mass formulation:
    consistent_no_rotation_inertia=0;
    consistent_with_rotation_inertia=1;
    lumped_diagonal_no_rotation_inertia=2;
    lumped_diagonal_with_rotation_inertia=3;
    mass_type=1;
    
    %%
    % Properties
    prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    
    %%
    % Finite element block
    feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells,...
        'mass_type',mass_type));
    
    %%
    % Initialize configuration variables
    geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
    u0=0*geom0;
    Rfield0 = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
    % Define the displacement increment field
    dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));
    
    % Apply EBC's
    ebc_fenids=[clampedn];
    ebc_prescribed=[1];
    ebc_comp=[];
    ebc_val=0;
    dchi = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    dchi = apply_ebc (dchi);
    % Number equations
    dchi = numbereqns (dchi);
    neqns=get(dchi, 'neqns');
    
    
    if (graphics)
        gv=graphic_viewer;
    else
        hold on;
    end
    
    % Now comes the nonlinear solution
    dchi = dchi*0; % zero out the displacement
    v0 = dchi*0; % zero out the Velocity
    xyz = gather(geom0,(1:count(fens)),'values','noreshape');
    V0Values = gather(v0,(1:count(fens)),'values','noreshape');
    xyz = gather(geom0,(1:count(fens)),'values','noreshape');
    for j=1:size(xyz,1 )
        if (xyz(j,3)<L/10)
            V0Values(j,2)= vmagn*(L/10-xyz(j,3))/(L/10);
        end
    end
    v0 = scatter(v0,(1:count(fens)),V0Values);
    a0 = dchi*0; % zero out the Acceleration
    utol = 1e-13*get(dchi, 'neqns');
    
    
    K = start (sparse_sysmat, get(dchi, 'neqns'));
    M = start (sparse_sysmat, get(dchi, 'neqns'));
    G = start (sparse_sysmat, get(dchi, 'neqns'));
    
    Tip_uy= [];
    t=0; %
    step=0;
    while (t <= tend)
        t = t + dt;
        Report(['Time ' num2str(t) ]); % pause
        % Initialization
        dchi = apply_ebc(dchi);% Apply boundary conditions
        u1 = u0; % guess
        Rfield1=Rfield0; % guess
        stepdchi = 0*apply_ebc(dchi);% Total increment in current step
        a1 = -(1/Newmarkb/dt)*v0 -(1/2-Newmarkb)/Newmarkb*a0;
        v1 = v0 + dt*((1-Newmarkg)*a0 + Newmarkg*a1);
        dchipv =gather_sysvec(dt*v0 +(dt^2/2*(1-2*Newmarkb))*a0);
        vpv =gather_sysvec(v0 +(dt*(1-Newmarkg))*a0);
        
        iter=1;
        while 1
            feb1=feb;
            F = start (sysvec, get(dchi, 'neqns'));
            [feb,evs] = restoring_force(feb,geom0,u1,Rfield1,dchi);       % Internal forces
            F = assemble (F, evs);
            K =assemble (K,cat(2,stiffness(feb,geom0,u1,Rfield1,dchi),stiffness_geo(feb,geom0,u1,Rfield1,dchi)));
            Km=get(K, 'mat');
            M =assemble (M,mass(feb,geom0,u1,Rfield1,dchi));
            Mm=get(M, 'mat');
            G =assemble (G,coriolis(feb,geom0,u1,Rfield1,v1,dchi));
            Gm=get(G, 'mat');
            rhs=get(F,'vec')...
                +Mm*((-1/(Newmarkb*dt^2))*gather_sysvec(stepdchi)+(1/(Newmarkb*dt^2))*dchipv)...
                +Gm*((-Newmarkg/Newmarkb/dt)*gather_sysvec(stepdchi)+(Newmarkg/Newmarkb/dt)*dchipv-vpv);
            dchi = scatter_sysvec(dchi, (Km+(Newmarkg/Newmarkb/dt)*Gm+(1/(Newmarkb*dt^2))*Mm)\rhs); % Displacement increment
            u1 = u1 + slice(dchi,(1:3),'transl');   % increment displacement
            stepdchi = stepdchi + dchi;
            v1 = v1 + (Newmarkg/Newmarkb/dt)*dchi;
            a1 = a1 + (1/Newmarkb/dt^2)*dchi;
            Rfield1=update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat'));
            %             Report(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
            if (max(abs(gather(dchi,[],'values'))) < utol) break; end;% convergence check
            if (iter >maxit)% bailout for failed convergence
                error( [' Possible failed convergence']);
            end
            K = reinitialize (K);
            M = reinitialize (M);
            G = reinitialize (G);
            iter=iter+1;
        end
        [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);        % final update
        %         Report(['    Converged for t=' num2str(t)]); % pause
        u0 = u1;       % update the displacement
        Rfield0=Rfield1; % update the rotations
        v0 = v1;       % update the velocities
        a0 = a1;       % update the accelerations
        
        if graphics
            gv=reset(gv,[]);
            draw(feb, gv, struct ('x', geom0, 'u', 0*u1, 'R', Rfield0, 'facecolor','none'));
            draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','g'));
            %         draw_local_coordinates(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'length',12.5));
            %             camset(gv, camera);
            drawnow;
        else
            tipu=gather(u1,tipn, 'values', 'noreshape');
            Tip_uy=[Tip_uy;tipu(2)];
            plot(t,Tip_uy(end),'k+','linewidth',3); hold on
            xlabel('Time');
            ylabel('Tip transverse displacement');
            set_graphics_defaults
            set(gca,'xlimmode','manual');
            %                 set(gca,'ylimmode','manual');
            %                 set(gca,'DataAspectRatioMode','auto');
            set(gca,'xlim', [0,tend]);
            %                 set(gca,'ylim',
            %                 1.2*[-10,+10]);
            grid on
            drawnow;figure(gcf);
        end
        step=step+1;
    end
end
