%%  Curved (45 deg circular arc) cantilever, Transverse force at the tip


%% 
% Large-deflection problem. Solved many times:
% 
% Bathe, Boulourchi 1979
% Simo, Vu-Quoc 1986
% Cardona, Geradin 1988
% Krysl 1993
% Krysl, FAESOR script curved_cantilever

% Present calculation refers to the data of
% Reference: EULERIAN FORMULATION FOR LARGE-DISPLACEMENT ANALYSIS OF SPACE
% FRAMES, by B.  A.  lzzuddin I and A.  S.  Elnashai,  
% Journal of Engineering Mechanics, Vol.  119, No.  3,  March, 1993. 

%% 
%
% Parameters:
E=1e7;% lb.in^-2
nu=0.3;
h= 1; b =1;% in
Parameters=Beam_cross_section(struct('type','square','a',h));
Fmag=600;%  lb
radius=100;% in
ang=45;
nel=8;% number of elements
utol=1e-6;

% Mesh
xyz=[];
id=[];
for i=0:nel
    id(end+1)=i+1;
    xyz=[xyz;[radius*cos(i/nel*ang/360*2*pi) radius*sin(i/nel*ang/360*2*pi) 0]];
end
fens=fenodeset(struct ('id',id,'xyz',xyz));
tipn=nel+1;

id=[];
conns = [];
for i=1:nel
    id(end+1)=i; conns= [conns;[i i+1]];
end
    Parameters.id=id;
    Parameters.conn=conns;
    Parameters.x1x2_vector=[0,0,1];
gcells = gcellset_beam3(Parameters);
    

% Material
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));
% Configuration variables
geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
    u=0*geom0;
    Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
    Rfield0=Rfield;
    % Define the displacement increment field
    dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));

% Apply EBC's
ebc_fenids=[1];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=ebc_fenids*0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
dchi  = apply_ebc (dchi);
% Number equations
dchi  = numbereqns (dchi);
neqns=get(dchi, 'neqns');

gv=graphic_viewer;
    

load_parameters=0.05:0.05:1;
tip_displacement=[];
for load_parameter=load_parameters
    
    % Initialization
    dchi = 0*dchi; % guess
    dchi = apply_ebc(dchi);
    u1 = u;
    Rfield1=Rfield;
    nl = nodal_load(struct ('id',tipn,'dir',[3],'magn',[load_parameter*Fmag]));
    
    iter=1;
    while 1
        feb1=feb;  % Make a copy of the state
        F = start (sysvec, neqns);
        F = assemble (F, loads(nl, dchi));
        [feb1,evs] = restoring_force(feb1,geom0,u1,Rfield1,dchi);       % Internal forces
        F = assemble (F, evs);
        K = start (sparse_sysmat, neqns);
        K = assemble (K, cat(2,stiffness(feb1,geom0,u1,Rfield1,dchi),stiffness_geo(feb1,geom0,u1,Rfield1,dchi)));
        dchi = scatter_sysvec(dchi, K\F); % Displacement increment
        % Line search
        R0 = dot(get(F,'vec'),gather_sysvec(dchi));
        F = start (sysvec, neqns);
        F = assemble (F, loads(nl, dchi));
        [feb1,evs] = restoring_force(feb1,geom0,...
            u1+slice(dchi,(1:3),'transl'),update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat')),dchi);       % Internal forces
        F = assemble (F, evs);
        R1 = dot(get(F,'vec'),gather_sysvec(dchi));
        a = R0/R1;
        if ( a<0 )
            eta = a/2 +sqrt((a/2)^2 -a);
        else
            eta =a/2;
        end
        eta=min( [eta, 1.0] );% line search concluded
        u1 = u1 + eta*slice(dchi,(1:3),'transl');   % increment displacement
        Rfield1=update_rotation_field(Rfield1,eta*slice(dchi,(4:6),'rotat'));
        disp(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
        
        draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','none'));
  camset (gv, [-392.3967 -581.0903  474.9767   71.8588   23.9391   34.6762 0         0    1.0000    3.4650]);
   pause(0.1)
  
        if (max(abs(gather(dchi,[],'values'))) < utol) break; end;                    % convergence check
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);        % final update
    disp(['    Converged for load_parameter=' num2str(load_parameter)]); % pause
    u = u1; % update the displacement
    Rfield=Rfield1; % update the rotations
    utip=gather(u,get(nl, 'id'),'values');
    disp(['Tip deflection =' num2str(utip(1:3)','%27.15e, %27.15e, %27.15e\n')])
    tip_displacement=[tip_displacement;utip(1:3)'];
    

end



%% 
% Comparison with reference solution.

izzelna_ux =[    0.0317    0.6048
   -0.0628    0.6048
    8.7273   34.7095
   16.6668   85.8664
   23.7557  134.8918
   32.6404  213.7587
   37.8389  264.9157
   41.7141  318.2041
   44.9277  382.1503
   47.9523  446.0965
   50.4098  510.0426
   51.9220  578.2519
   53.9069  657.1188
   53.9069  657.1188];

izzelna_uy =[-0.1574    0.6048
    0.0317    2.7364
    1.1659   73.0772
    2.7727  132.7603
    4.9466  211.6272
    7.1205  284.0995
    8.9164  350.1772
   10.2396  414.1234
   11.3738  480.2011
   13.1697  548.4104
   14.0203  618.7511
   14.6820  657.1188
   14.5875  657.1188];




izzelna_uz =[   -0.5354    2.7364
   -0.2519    2.7364
   -1.1970   51.7618
   -2.2367   90.1295
   -4.3161  124.2341
   -6.8681  183.9172
   -9.1366  235.0741
  -11.8776  288.3626
  -14.5241  354.4403
  -16.9815  411.9919
  -19.3445  475.9380
  -21.7074  550.5419
  -23.9759  618.7511
  -25.3936  661.3819];


figure
plot(izzelna_ux(:,1),izzelna_ux(:,2),'r-','linewidth',3)
hold on
plot(izzelna_uy(:,1),izzelna_uy(:,2),'r-','linewidth',3)
hold on
plot(izzelna_uz(:,1),izzelna_uz(:,2),'r-','linewidth',3)
hold on
plot(tip_displacement(:,1),load_parameters*Fmag,'x','linewidth',3)
plot(tip_displacement(:,2),load_parameters*Fmag,'x','linewidth',3)
plot(tip_displacement(:,3),load_parameters*Fmag,'x','linewidth',3)
title('Curved cantilever, L-to-R uy,ux,uz','fontname','Times','fontsize',16)
labels(' Tip displacement [in]','Load [lb]')
grid on
set_graphics_defaults

