% Transient simulation to demonstrate the smearing out of the initial
% temperature spike 

kappa=[0.2 0; 0 0.2]; % conductivity matrix
Q=0; % uniform heat source
theta = 1; % generalized trapezoidal method
dt=10; % time step
tend= 20*dt; % length of the time interval
t=0;
online_graphics=1;
[fens,gcells] = targe2_mesher({... 
        ['curve 1 line -30 -20 30 -20'],...
        ['curve 2 line 30 -20 30 20'],...
        ['curve 3 line 30 20 -30 20'],...
        ['curve 4 line -30 20 -30 -20'],...
        'subregion 1  property 1 boundary 1 2 3 4 ',...
        ['m-ctl-point constant 2']
        }, 1.0);
prop=property_diffusion(struct('conductivity',kappa,'source',Q));
mater=mater_diffusion (struct('property',prop));
feb = feblock_diffusion (struct ('mater',mater,...
    'gcells',gcells,...
    'integration_rule',tri_rule(1)));
geom = field(struct('name',['geom'], 'dim', 2, 'fens',fens));
tempn=field(struct('name',['tempn'], 'dim', 1, 'nfens',...
    get(geom,'nfens')));

tempn = numbereqns (tempn);
xy=get(geom,'values');
T0=500./(xy(:,1).^2+xy(:,2).^2+5);
tempn = set(tempn,'values',T0);

K = start (sparse_sysmat, get(tempn, 'neqns'));
K = assemble (K, conductivity(feb, geom, tempn));
Km = get(K,'mat');
C = start (sparse_sysmat, get(tempn, 'neqns'));
C = assemble (C, capacity(feb, geom, tempn));
Cm = get(C,'mat');

if online_graphics
    gv=graphic_viewer;
    set_graphics_defaults(gcf)
    axis equal vis3d
end
minmaxT = []
while t<tend+0.1*dt % Time stepping
    if online_graphics
        gv=reset (gv,struct('limits',[-30,30,-20,20,0,100]));
        T=get(tempn,'values');
        geomT=field(struct ('name', ['geomT'], ...
            'data',[get(geom,'values'), get(tempn,'values')]));
        draw(gcells, gv, struct ('x',geomT, 'u',0*geomT,...
            'facecolor','y'));
        camset(gv,[-146.1721 -410.9993  485.8627  -13.8460   -3.7415   42.4319         0         0 1.0000    8.7074]);
        figure(gcf);
        title (['Time =' num2str(t)]); hold off; pause(0.1);
%                 saveas(gcf, ['shrinkfit-' num2str(t) '.png'], 'png');
    end
    tempn1 = tempn;
    F = start (sysvec, get(tempn, 'neqns'));
    Tn=gather_sysvec(tempn);
    %     minmaxT= [minmaxT; min(Tn),max(Tn)] 
    Tn1 = (1/dt*Cm+theta*Km) \ ((1/dt*Cm-(1-theta)*Km)*Tn+...
        get(F,'vec'));
    tempn = scatter_sysvec(tempn1,Tn1);
    t=t+dt;
end
