kappa=[1.8 0; 0 1.8]; % conductivity matrix
qn=113; % heat flux
Ri=0.5;
Ro=0.55;
num_integ_pts=1; % 1-point quadrature
fens = fenodeset(struct('id',1:4','xyz',[[Ri*cos(0*pi/180),Ri*sin(0*pi/180)];...
    [Ro*cos(0*pi/180),Ro*sin(0*pi/180)];...
    [Ro*cos(10*pi/180),Ro*sin(10*pi/180)];...
   [Ri*cos(10*pi/180),Ri*sin(10*pi/180)]]));
gcells = gcellset_T3(struct('id',1:2,'conn',[[1,2,3];[3,4,1]],'other_dimension', 1));
bgcells = gcellset_L2(struct('id',1,'conn',[2,3],'other_dimension', 1));
prop=property_diffusion(struct('conductivity',kappa,'source',0));
mater=mater_diffusion (struct('property',prop));
feb = feblock_diffusion (struct ('mater',mater,...
    'gcells',gcells,...
    'integration_rule',tri_rule(num_integ_pts)));
efeb = feblock_diffusion (struct (...
    'mater',mater,...
    'gcells',bgcells,...
    'integration_rule',gauss_rule(1,2)));
geom = field(struct('name',['geom'], 'dim', 2, 'fens',fens));
theta=field(struct('name',['theta'], 'dim', 1, 'nfens',...
    get(geom,'nfens')));
fenids=[fenode_select(fens,struct('distance',Ri,'from',[0 0],...
    'inflate', 0.01))];
prescribed=ones(length(fenids),1);
comp=[];
val=zeros(length(fenids),1)+17;
theta = set_ebc(theta, fenids, prescribed, comp, val);
theta = apply_ebc (theta);
theta = numbereqns (theta);
K = start (sparse_sysmat, get(theta, 'neqns'));
K = assemble (K, conductivity(feb, geom, theta));
F = start (sysvec, get(theta, 'neqns'));
fi= force_intensity(struct('magn',qn));
F = assemble (F, flux_loads(efeb, geom, theta, fi));
F = assemble(F, nz_ebc_loads_conductivity(feb, geom, theta));
theta = scatter_sysvec(theta, get(K,'mat')\get(F,'vec'));
% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
T=get(theta,'values');
dcm=data_colormap(struct('range',[min(T),max(T)],'colormap',hot));
colorfield=field(struct ('name', ['colorfield'], 'data',...
    map_data(dcm, T)));
geomT=field(struct ('name', ['geomT'], ...
    'data',[get(geom,'values'), (get(theta,'values'))]));
draw(gcells, gv, struct ('x',geomT, 'u',0*geomT,...
        'colorfield',colorfield, 'shrink',1.0));
    draw(gcells, gv, struct ('x',geom, 'u',0*geom, ...
        'facecolor','none'));
axis tight% vis3d
set(gca,'DataAspectRatio', [1, 1, 5*(max(T)/Ro)])
zlabel('Temperature [degree Celsius]')
(get(theta,'values'))