function Arch_buckling_H27
disp('Example: linear Cylindrical arch buckling analysis, H27 elements');
% Parameters:
E=71240.0;%MPa
nu=0.31;% Poisson ratio
rA=pi/2 ; Width= 30; Thickness= 0.6; 
nc= 22; nr= 4; na=1;
iR = 240;
magn=1e-5/30/0.6;
scale=15;
% magn_cr = E*min([H,W])^3*max([H,W])/12*pi^2/(2*L)^2/W/H;
neigvs = 4;

% Mesh

[fens,gcells]=H8_block(rA,Width,Thickness,nc,nr,na);
[fens,gcells] = H8_to_H27(fens,gcells); 
xyz=get (fens,'xyz');
for i=1:count (fens)
    x=xyz(i,1); y=xyz(i,2); z=xyz(i,3);
    xyz(i,:)= [0, y-iR-Width/2, z];
    xyz(i,:) = xyz(i,:)*rotmat([0, 0, -x]);
end
fens=set(fens,'xyz', xyz);
% drawmesh({fens,gcells},'gcells')
% Material
prop = property_linel_iso(struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax(struct('property',prop));
% Finite element block
feb = feblock_defor_ss_nonlinear (struct ('mater',mater, 'gcells',gcells,...
    'integration_rule',gauss_rule(3,3)));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = 0*geom; % zero out

% Apply EBC's
ebc_fenids=fenode_select (fens,struct('box',[0,0,-Inf,Inf,-Inf,Inf,],'inflate', Thickness/1000));
ebc_prescribed=ones(1, length(ebc_fenids));
ebc_comp=[];
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);
% Assemble the system matrix
ems = stiffness(feb, geom, u, u);
K = start (sparse_sysmat, get(u, 'neqns'));
K = assemble (K, ems);

% Load
bdry_gcells = mesh_bdry(gcells, []);
bcl = gcell_select(fens, bdry_gcells, ...
    struct ('box',[-Inf,Inf,0,0,-Inf,Inf],'inflate',Width/1000));
lfeb = feblock_defor_ss(struct ('mater',mater, 'gcells',subset(bdry_gcells,bcl),...
    'integration_rule',gauss_rule(2,2)));
fi= force_intensity(struct ('magn',[-magn;0;0]));
F = start (sysvec, get(u, 'neqns'));
F = assemble (F, distrib_loads(lfeb, geom, u, fi,2));
% Solve
u = scatter_sysvec(u, K\F);


ulin = u; %  linear displacement solution

disp('Stability analysis');
[feb1,evs] = restoring_force(feb,geom,ulin,0*ulin);       % Internal forces
ems = stiffness_geo(feb1, geom, ulin, ulin);            % Geometric stiffness
K_geo = start (sparse_sysmat, get(u, 'neqns'));
K_geo = assemble (K_geo, ems);

options.tol =1e-26;
options.maxit = 5000;
options.disp = 0;
[W,Omega]=eigs(-(get(K_geo,'mat')+get(K_geo,'mat')')/2,(get(K,'mat')+get(K,'mat')')/2,...
    neigvs,'lm', options);
diag(Omega)
%         [Omegas,ix]=sort(1./diag(Omega)) ;
Omegas=(1./(diag(Omega)))
% [W,Omega]= eig(full(get(K,'mat')),full(get(K_geo,'mat')));
% Omegas =diag(Omega);
% % ix= 1:size(Omega,1)
% [Ignore,ix] =sort (abs(diag(Omega)));
% W=W(:,ix);
% Omegas =Omegas(ix);
% ix =find(Omegas<0);

gv=graphic_viewer;
cmap = jet(16);
w=clone(ulin,'w');
for i=neigvs:-1:1
    disp(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
    gv=reset(clear(gv,[]),[]);
            
    w = scatter_sysvec(w, W(:,i));
    v = get(w,'values');
    wmag = field(struct ('name',['wmag'], 'dim', 1, 'nfens',count(fens)));
    wmag = scatter(wmag,1:get(wmag,'nfens'),sqrt(v(:,1).^2+v(:,2).^2+v(:,3).^2));
    nvals =gather(wmag,1:get(wmag,'nfens'),'values');
    nvalsrange=[min(nvals),max(nvals)];
    dcm=data_colormap(struct ('range',nvalsrange, 'colormap',cmap));
    colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
    draw(feb,gv, struct ('x', geom, 'u', scale*w,'colorfield',colorfield, 'shrink',1.0));

    pause(1); % next eigenvector
end

