function perforated_strip2
    disp('Example Perforated_strip: small strain nonlinear analysis');
    % Parameters:
    E=7000;
    nu=0.3;
    sigma_y=24.3;
    L= 360; % Length of the beam
    W = 10; % Thickness of the plate
    H = 200; % Width
    R= 100;
    nL=8;nH=4;nR=4;nT=2;
    %     nL=3;nH=1;nR=1;
    umag=2.5;
    scale=10;
    stressscale=scale/20;
    epscale=5*scale;
    nincr =15;
    utol = 10e-7;
    graphics = true;

    % Mesh
    [fens,gcells]=Q4_elliphole(R,R,L,H,nL,nH,nR,[]);
    [fens,gcells] = Q4_extrude(fens,gcells,nT,@(x,i)([x,0]+[0,0,W*i/nT]));
    [fens,gcells] = H8_to_H20(fens,gcells);
    % Material
    prop = property_plastic_perf_j2_iso(struct('E',E,'nu',nu,'sigma_y',sigma_y));
    mater = mater_ss_plastic_perf_j2_triax(struct('property',prop));
    % Finite element block
    feb = feblock_defor_ss_nonlinear (struct ('mater',mater, 'gcells',gcells,...
        'integration_rule',gauss_rule(3,3)));
    % Geometry
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    % Define the displacement field
    u   = 0*geom; % zero out
    Center_fenids=fenode_select (fens,struct('box',[L,L,W/2, W/2,-Inf,Inf],'inflate',1/1000));
    % Apply EBC's
    ebc_fenids=fenode_select (fens,struct('box',[0,0,-Inf,Inf,-Inf,Inf],'inflate',1/1000));
    ebc_prescribed=ones(1,length (ebc_fenids));
    ebc_comp=ones(1,length (ebc_fenids))*1;
    ebc_val=ebc_fenids*0;
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    ebc_fenids=fenode_select (fens,struct('box',[-Inf,Inf,0,0,-Inf,Inf],'inflate',1/1000));
    ebc_prescribed=ones(1,length (ebc_fenids));
    ebc_comp=ones(1,length (ebc_fenids))*2;
    ebc_val=ebc_fenids*0;
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    ebc_fenids=fenode_select (fens,struct('box',[-Inf,Inf,-Inf,Inf,0,0],'inflate',1/1000));
    ebc_prescribed=ones(1,length (ebc_fenids));
    ebc_comp=ones(1,length (ebc_fenids))*3;
    ebc_val=ebc_fenids*0;
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    ebc_fenids=fenode_select (fens,struct('box',[L,L,-Inf,Inf,-Inf,Inf],'inflate',1/1000));
    ebc_prescribed=ones(1,length (ebc_fenids));
    ebc_comp=ones(1,length (ebc_fenids))*1;
    ebc_val=ebc_fenids*0+  0*umag;
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    u   = apply_ebc (u);
    % Number equations
    u   = numbereqns (u);
    % Assemble the system matrix
    ems = stiffness(feb, geom, u, u);
    K = start (dense_sysmat, get(u, 'neqns'));
    K = assemble (K, ems);
    
    
    % Now comes the nonlinear solution
    tup = 1;
    u = u*0; % zero out the displacement

    if (graphics)gv=graphic_viewer;end

    Center_deflection = [];
    t=0; % time=load magnitude
    incr=1;
    while (incr <= nincr)
        t = t + tup / nincr;
        % Initialization
        u1 = u; % guess
        u1 = apply_ebc(u1);
        du = 0*u; % this will hold displacement increment
        du = apply_ebc(du);
        iter=1;
        du0norm = [];
        while 1
            feb1=feb;                                           % Make a copy of the state
            ebc_fenids=fenode_select (fens,struct('box',[L,L,-Inf,Inf,-Inf,Inf],'inflate',1/1000));
            ebc_prescribed=ones(1,length (ebc_fenids));
            ebc_comp=ones(1,length (ebc_fenids))*1;
            ebc_val=ebc_fenids*0+ t*umag;
            u1 = set_ebc(u1, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
            u1 = apply_ebc (u1);
            F = start (sysvec, get(u1, 'neqns'));
%             F = assemble (F, nz_ebc_loads(feb1, geom, u1));
            [Ignore,evs] = restoring_force(feb1,geom,u1,u);       % Internal forces
            F = assemble (F, evs);
            K = start (sparse_sysmat, get(u, 'neqns'));
            K = assemble (K, stiffness(feb1, geom, u1,u));
            du = scatter_sysvec(du, K\F);
            R0 = dot(get(F,'vec'),gather_sysvec(du));
            F = start (sysvec, get(u1, 'neqns'));
            [Ignore,evs] = restoring_force(feb1,geom,u1+du,u);       % Internal forces
            F = assemble (F, evs);
            R1 = dot(get(F,'vec'),gather_sysvec(du));
            a = R0/R1;
            if ( a<0 )
                eta = a/2 +sqrt((a/2)^2 -a);
            else
                eta =a/2;
            end
            eta=min( [eta, 1.0] )
            u1 = u1 + eta*du;                                       % increment displacement
            dunorm =norm(du);
            if (isempty(du0norm)) du0norm =dunorm;end
            disp(['   Iteration ' num2str(iter) ': ||du||/||du0||=' num2str(dunorm/du0norm)]);
            if (graphics)
                 gv=reset(clear(gv,[]),[]);
                camset (gv,1.0e+003 *[ -2.1416   -1.4296    3.3375    0.1981    0.1191   -0.0063    0.0006    0.0004    0.0006 0.0039]);
                draw(feb,gv, struct ('x', geom, 'u', +0*u1,'facecolor','none', 'edgecolor','red'));
                draw(feb,gv, struct ('x', geom, 'u', +scale*u1,'facecolor','none', 'edgecolor','black'));
draw_axes (gv, [])
                drawnow; pause(0.1)
            end
            Center_u =gather (u,Center_fenids,'values','noreshape');

            if (dunorm < utol*du0norm) break; end;                    % convergence check
            iter=iter+1;
        end
        [feb,ignore] = restoring_force(feb1,geom,u1,u);        % final update
        u = u1;                                               % update the displacement
        if graphics
            gv=reset(clear(gv,[]),[]);
            camset (gv,1.0e+003 *[ -2.1416   -1.4296    3.3375    0.1981    0.1191   -0.0063    0.0006    0.0004    0.0006 0.0039]);
            fld = field_from_integration_points(feb, geom, u1, [], 'equiv_pl_def',1);           
            dcm=data_colormap(struct ('range',[0,0.005], 'colormap',jet));
            colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, get(fld,'values'))));
            draw(feb,gv, struct ('x', geom,'u',scale*u, 'colorfield',fld));
            %             draw_integration_points(feb,gv,struct ('x',geom,'u',u,'u_displ',scale*u, 'scale',epscale,'quantity',['equiv_pl_def'],'component',1,'data_cmap', dcm));
            title (['Increment ' num2str(incr) ': Converged for t=' num2str(t) ])
            drawnow; pause(0.1)
        end
        disp(['Increment ' num2str(incr) ': Converged for t=' num2str(t) ]); % pause
        Center_deflection = [Center_deflection,mean(Center_u(:,3))];
        incr = incr + 1;
    end
    Center_deflection
%     camget (gv)
end

function id= mn(id,out,xyz,pc)
    id.container=min(out(id.comp), id.container);
end

function id= mx(id,out,xyz,pc)
    id.container=max(out(id.comp), id.container);
end
%draw(gmlin, 'dataidx', 3, 'scale', scale, 'displace_by', [1 2 3], 'facecolor', 'none')