%%  Define some auxiliary variables
% clear all classes;
E=1000;
nu=0.4999999;
nu=0.499999;
W=2.5;
H=5;
L= 50;
htol=min([L,H,W])/1000;
uzex =-12.6;
magn = 0.2*uzex/4;
Force =magn*W*H*2;
% Force*L^3/(3*E*W*H^3*2/12)
graphics = ~true;

n_per_length=4;
eix=1;

% regular tetrahedron
eltyd(eix).legend ='T4';
eltyd(eix).mf =@T4_block;
eltyd(eix).ns=(1:8);
eltyd(eix).blf =@feblock_defor_ss;
eltyd(eix).integration_rule=tet_rule (1);
eltyd(eix).surface_integration_rule=tri_rule(1);
eltyd(eix).styl='kv-';
eix=eix+1;

% Selective reduced integration hexahedron
eltyd(eix).legend ='H8-SRI';
eltyd(eix).mf =@H8_block;
eltyd(eix).ns=(1:8);
eltyd(eix).blf =@feblock_defor_ss_sri;
eltyd(eix).integration_rule=gauss_rule (3,[1,2]);
eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
eltyd(eix).styl='md-';
eix=eix+1;

% Full integration hexahedron
eltyd(eix).legend ='H8';
eltyd(eix).mf =@H8_block;
eltyd(eix).ns=(1:8);
eltyd(eix).blf =@feblock_defor_ss;
eltyd(eix).integration_rule=gauss_rule (3,2);
eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
eltyd(eix).styl='mx--';
eix=eix+1;

% Reduced integration H20
eltyd(eix).legend ='H20R';
eltyd(eix).mf =@H20_block;
eltyd(eix).ns=(1:5);
eltyd(eix).blf =@feblock_defor_ss;
eltyd(eix).integration_rule=gauss_rule (3,2);
eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
eltyd(eix).styl='ro-';
eix=eix+1;

% Full integration H20
eltyd(eix).legend ='H20R';
eltyd(eix).mf =@H20_block;
eltyd(eix).ns=(1:5);
eltyd(eix).blf =@feblock_defor_ss;
eltyd(eix).integration_rule=gauss_rule (3,3);
eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
eltyd(eix).styl='b*-';
eix=eix+1;

% T10 tetrahedron
eltyd(eix).legend ='T10';
eltyd(eix).mf =@T10_block;
eltyd(eix).ns=(1:4);
eltyd(eix).blf =@feblock_defor_ss;
eltyd(eix).integration_rule=tet_rule (4);
eltyd(eix).surface_integration_rule=tri_rule(3);
eltyd(eix).styl='gd-';
eix=eix+1;

all_ns={};
all_uzs={};
for eix = 1:length(eltyd)
    ns=[];
    uzs=[];
    
    for     n=eltyd(eix).ns;
        mult =n_per_length;
        %% Create the mesh and initialize the geometry
        [fens,gcells]= feval (eltyd(eix).mf, W, L, H,n,mult*n,2*n);
        prop = property_linel_iso (struct('E',E,'nu',nu));
        mater = mater_defor_ss_linel_triax (struct('property',prop));
        
        feb = feval (eltyd(eix).blf, struct ('mater',mater, 'gcells',gcells, 'integration_rule',eltyd(eix).integration_rule));
        geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
        
        %% Define the displacement field, and zero it out
        u   = 0*geom;
        
        %% Apply the EBC''s
        ebc_fenids=fenode_select(fens,struct('box',[0 W 0 0 0 H]));
        ebc_prescribed=ones(length(ebc_fenids),1);
        ebc_comp=[];
        ebc_val=zeros(length(ebc_fenids),1);
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        ebc_fenids=fenode_select(fens,struct('box',[W W 0 L 0 H]));
        ebc_prescribed=ones(length(ebc_fenids),1);
        ebc_comp=ebc_fenids*0+1;
        ebc_val=zeros(length(ebc_fenids),1);
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        u   = apply_ebc (u);
        
        %% Number the equations
        u   = numbereqns (u);
        disp (['n=' num2str(n) ', number of equations =' num2str(get(u,'neqns'))])
        
        %% Assemble the stiffness matrix
        K = start (sparse_sysmat, get(u, 'neqns'));
        K = assemble (K, stiffness(feb, geom, u));
        
        %% Assemble the load vector
        fi= force_intensity(struct ('magn',[0;0; magn]));
        bdry_gcells = mesh_bdry(gcells, []);
        bcl = gcell_select(fens, bdry_gcells, ...
            struct ('box',[0 W L L 0 H],'inflate',htol));
        lfeb = feblock_defor_ss(struct ('mater',mater, 'gcells',subset(bdry_gcells,bcl),...
            'integration_rule',eltyd(eix).surface_integration_rule));
        F = start (sysvec, get(u, 'neqns'));
        F = assemble (F, distrib_loads(lfeb, geom, u, fi,2));
        
        %% Solve K*u=F
        x=K\F;
        
        %% Transfer the solution to the field u
        u = scatter_sysvec(u, x);
        ebc_fenids=fenode_select(fens,struct('box',[0 W L L 0 H]));
        uv=gather (u,ebc_fenids,'values','noreshape');
        uz=sum(uv(:, 3))/length(ebc_fenids);
        disp (['   Free end displacement =' num2str(uz)])
        ns=[ns get(u,'neqns')];
        uzs =[uzs uz];
        
        %% Plot the displacement field u
        if graphics
            gv=graphic_viewer;
            gv=reset (gv,[]);
            scale=1;
            % draw(feb,gv, struct ('x', geom,'u',0*u, 'facecolor','none'));
            fld = field_from_integration_points(feb, geom, u, [], 'Cauchy', 2);
            nvals=get(fld,'values');
            dcm=data_colormap(struct ('range',[min(nvals),max(nvals)], 'colormap',jet));
            colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
            draw(feb, gv, struct ('x',geom,'u', scale*u, 'colorfield',colorfield, 'shrink',1));
            colormap(jet);
            cbh=colorbar;
            set(cbh,...
                'Position',[0.8 0.15 0.1 0.7],...
                'YLim',[0,1],...
                'YTick',[0,1],...
                'YTickLabel',{[num2str(min(nvals))],[num2str(max(nvals))]});%{[num2str((min(nvals)))],[num2str((max(nvals)))]}
            set(get(cbh,'XLabel'),'String','\sigma_{y}');
            title ('Stress');
            camset (gv, [362.6179  179.7380  186.8838    4.5277   24.0360   -3.5642   -0.4020   -0.1748  0.8988    3.9476])
        else
            plot(ns,uzs/uzex,eltyd(eix).styl,'linewidth',3); hold on
            figure (gcf); pause (1)
        end
    end
    all_ns{end+1}=ns;
    all_uzs{end+1}=uzs;
end

figure;
legends={};
for eix = 1:length(eltyd)
    legends{end+1}= eltyd(eix).legend;
    plot(all_ns{eix},all_uzs{eix}/uzex,eltyd(eix).styl,'linewidth',3); hold on
end
legend(legends); grid on
labels( 'Number of equations', 'Normalized deflection');
set_graphics_defaults; set_label_defaults
figure (gcf);


figure;
legends={};
for eix = 1:length(eltyd)
    legends{end+1}= eltyd(eix).legend;
    ns=    all_ns{eix}; uzs=all_uzs{eix};
    loglog(ns(2:end),abs(diff(uzs)),eltyd(eix).styl,'linewidth',3); hold on
end
legend(legends); grid on
labels( 'Number of equations', 'Approximate error of the deflection');
set_graphics_defaults; set_label_defaults
figure (gcf);

% results= {[ -3.902394552423855  -7.944439653004491  -9.892350605190988 -10.836725525178135],
% [ -0.094725342155956  -0.096371776253977  -0.099480972557935  -0.102941413172421],
% [ -11.517387569099283 -11.951906216695427 -12.091601377770402 -12.168805962027948],
% [  -9.514689395147927 -11.233013305870969 -11.716939680011308 -11.930196950238052]};
%
% result=0;
% for i=1:length(results)
%     all_uzs{i}
%     result=result+norm(results{i}-all_uzs{i})/length(results{i});
% end
% assignin('caller','faesor_test_passed',(result/length( results )<1e-3));