% Parameters:
u=physical_units_struct;
R= 0.5*u.M;
nref=8;
tol=R/1000;
    rhow = 1000*u.KG/u.M^3;
    bulkw = 2.1e9*u.PA;%
    Kstab=bulkw/1000000;
[1.43]*pi/R* sqrt(bulkw/rhow)/(2*pi)
fshift=2070;
OmegaShift=(2*pi*fshift);
neigvs = 80;

graphics = ~false;
tic
% Mesh
[fens,gcells] = T4_sphere(R,nref);
% [fens,gcells]=H8_sphere_n(R,nref);
%     [fens1,gcells1] = mirror_mesh(fens, gcells, [-1,0,0], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
%     [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
%     gcells=cat(gcells1,gcells2);
%     [fens1,gcells1] = mirror_mesh(fens, gcells, [0,-1,0], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
%     [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
%     gcells=cat(gcells1,gcells2);
%     [fens1,gcells1] = mirror_mesh(fens, gcells, [0,0,-1], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
%     [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
%     gcells=cat(gcells1,gcells2);
   
[fens, gcells]=renum_mesh(fens, gcells, 'symrcm');
 
% drawmesh({fens,gcells},'gcells','facecolor','red','shrink',0.85)
% Material
propw = property_viscous_fluid(struct('bulk',bulkw,'rho',rhow));
    matw = mater_defor_ss_linviscel_triax (struct('property',propw));
    febw = feblock_defor_ss(struct ('mater',matw,...
        'gcells',gcells,...
        'integration_rule',tet_rule(1)));
     febwirrot = feblock_defor_fluid_irrot_stab(struct ('mater',matw,...
       'gcells',gcells,...
        'integration_rule',tet_rule(1),'stabfact',Kstab));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
limits =reshape([min(get(geom,'values'));max(get(geom,'values'))], 6, 1);
% Define the displacement field
u   = 0*geom; % zero out
% No EBC's: free-floating
u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);

% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
evs=cat(2,stiffness(febw, geom, u),irrot_stab_stiffness(febwirrot, geom, u));
K = assemble (K, evs);
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(febw, geom, u);
% If lumped mass matrix, uncomment this block
mat=get(ems,'mat');
for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
    Me=mat{j};
    em2=sum(sum(Me));
    dem2=sum(diag(Me));
    mat{j}=diag(diag(Me)/dem2*em2);
end
ems=set(ems,'mat',mat);
% % Consistent or lumped mass matrix
M = assemble (M, ems);
disp(['Number of DOF =' num2str(get(u, 'neqns'))]);
toc
%
options.issym= true;
options.tol=1e-35;
options.maxit= 500;
options.disp= 2;
[W,Omega]=eigs(get(K,'mat')+OmegaShift^2*get(M,'mat'),get(M,'mat'),neigvs,'SM', options);
Omega =diag(diag(Omega)-OmegaShift^2);
[Ignore,ix] =sort (diag(Omega));
W=real(W);

for i=1:neigvs
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi+fshift) ' [Hz]']);
end


% Plot
if graphics
    Camera =1.0e+003 * [   -1.0612
        -0.8377
        1.0948
        -0.0775
        0.0025
        0.0093
        0
        0
        0.0010
        0.0033];
    xyz=get(geom,'values');
    box =reshape([min(xyz);max(xyz)],1,6);
    annloc=[mean(box(1:2)), mean(box(3:4))-mean(box(1:2)), mean(box(5:6))];
    bg=mesh_bdry(gcells);
    bfeb = feblock(struct ('gcells',bg));
    gv=graphic_viewer;
    gv=reset (gv,struct('limits',  limits));
    scale=.3;
    w=clone(u,'w'); % make a copy of u
    for i=neigvs:-1:7
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi+fshift) ' [Hz]']);
%         clf;
        w = scatter_sysvec(w, W(:,ix(i)));
        w = w*(1/norm(w ));
        wv = get(w,'values');
        wmag = real (sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2));
        dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
        colors=map_data(dcm, wmag);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
%         for xscale=scale*sin((0:1:42)/21*2*pi)
%             gv=reset (gv,[]);
% %             camset(gv,Camera);
%             draw(bfeb,gv, struct ('x', geom, 'u', xscale*w,'colorfield',colorfield));
%             draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]'],...
%                 struct('fontsize', 18));
%             draw_axes (gv,struct('length',R/20 ));
%             pause(0.005);
%         end
        %     saveas(gcf, ['twist_t4-' num2str(i) '.png'], 'png');
        pause(2); % next eigenvector
        gv=reset (gv,[]);
        %         camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom,'u',+scale*w,'colorfield',colorfield));
        draw(bfeb,gv, struct ('x', geom,'u',0*w,'facecolor','yellow','edgecolor',.53+[0, 0, 0],'alpha', 0.2));
        draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) '  [Hz]'],...
            struct('fontsize', 18));
        draw_axes (gv,struct('length',R/20 ));
        pause(2.55);
        
    end
end
