disp('Wave propagation along a beam');
% clear all classes
graphics = false;
% Parameters:
E=2.1e5;
sigma_y = 200;
nu=0.3;
rho= 7.85e-9;
L = 140;
W = 4;
H = 8;
vmag = 3000;
tfinal = 1e-2;
scale=10;
stressscale=scale/20;

% [fens,gcells] = block(L,W,H, 25, 2,2);
[fens,gcells] = block(L,W,H, 5, 1,3);
% [fens,gcells] = block(L,W,H, 5, 1,1);
[fens,gcells] = H8_to_H20(fens,gcells);
ir =gauss_rule (3,3);
% Material
prop = property_plastic_perf_j2_iso(struct('E',E,'nu',nu,'sigma_y',sigma_y,'rho',rho));
mater = mater_ss_plastic_perf_j2_triax(struct('property',prop));
% Finite element block
feb = feblock_ss_nonlinear (struct ('mater',mater, 'gcells',gcells,...
    'integration_rule',gauss_rule(3,3)));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = 0*geom; % zero out
% Apply EBC's

ebc_fenids=fenode_select (fens,struct('box',[L,L,-Inf,Inf,-Inf,Inf],'inflate',44/10000));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=[];
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select (fens,struct('box',[0,0,-Inf,Inf,-Inf,Inf],'inflate',44/10000));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=ebc_fenids*0 +1;
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);

% Number equations
u   = numbereqns (u);

tip_fenid=fenode_select (fens,struct('box',[0,0,0,0,0,0],'inflate',44/10000));
Tip_connectivity =gather(u,tip_fenid,'eqnums');



        
% Initial conditions
v = u;% copy everything from displacement field to velocity
fenids=fenode_select (fens,struct('box',[0,0,-Inf,Inf,-Inf,Inf],'inflate',44/10000));
for j=1:length(fens)
    xy= get (fens(j),'xyz');
    v = scatter(v,j,[0,0,-1]*vmag);
end
% Assemble the system matrix
ess=stiffness(feb, geom, u,u);% Current material stiffness
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(feb, geom, u);
Max_lambda = 0;
for j=1:length(ems)
    Me=get(ems(j),'mat');
    em2=sum(sum(Me));
    dem2=sum(diag(Me));
    Me =diag(diag(Me)/dem2*em2);
    ems(j)=set(ems(j),'mat',Me);
    Ke=get(ess(j),'mat');
%     [V,D] = eigs (Ke, Me, 1);
    [V,D] = eig (Ke, Me);
    Max_lambda = max( [Max_lambda,max(diag(D))]);
end
M = assemble (M, ems);
dt= 0.99* 2/sqrt(Max_lambda)
% Solve
Mmat = get(M,'mat');
U0 = gather_sysvec(u);
V0 = gather_sysvec(v);
u0 = scatter_sysvec(u, U0);
t=0;
F = start (sysvec, get(u, 'neqns'));
[feb,evs] = restoring_force(feb,geom,u0,u0);       % Internal forces
F = assemble (F, evs);
A0 = Mmat\(get(F,'vec'));
if graphics
    gv=graphic_viewer;
    gv=reset (gv,struct ('limits',[0 17 -26 26]));
end
ngraphics=  max (round(10*(0.3e-7)/dt),1);
igraphics =ngraphics-1;
snapshot =0;
while t <tfinal
    feb1 = feb;
    t=t+dt 
    U1 = U0 +dt*V0+(dt^2)/2*A0;
    u1 = scatter_sysvec(u, U1);
    F = start (sysvec, get(u, 'neqns'));
    [feb1,evs] = restoring_force(feb1,geom,u1,u);       % Internal forces
    F = assemble (F, evs);
    A1 = Mmat\(get(F,'vec'));
    V1 = V0 +dt/2* (A0+A1);
    U0 = U1;
    V0 = V1;
    A0 = A1;
    u0 = u1;
    
    igraphics=igraphics+1;
    if graphics & igraphics==ngraphics
        u = u1;
         nvals=get(slice(u, 3,'u'),'values');
        nvalsrange=[-2, 0];%[min(nvals),max(nvals)]
        dcm=data_colormap(struct ('range',nvalsrange, 'colormap',jet));
        colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
        clear(gv); gv=reset (gv,struct ('limits',[0-W L+W 0-W W+W 0-W H+W]));
        draw(feb,gv, struct ('x', geom, 'u', scale*u,...
            'colorfield',colorfield));%,'edgecolor','none'
        pause(0.01);
        %         saveas(gcf, [mfilename '-33-' num2str(snapshot) '.png'], 'png');
        snapshot = snapshot +1;
        igraphics= 0;
    else
%         plot (t,(V1'*Mmat*V1)/2,'bo'); hold on 
        plot (t,U1 (Tip_connectivity (3)),'bo'); hold on 
    drawnow
    end
end
