disp('Wave propagation along a beam');
% clear all classes
graphics = false;
% Parameters:
E=2.1e5;
sigma_y = 200;
nu=0.3;
rho= 7.85e-9;
L = 140;
W = 4;
H = 8;
vmag = 9130;
tfinal = 1e-2;
scale=10;
stressscale=scale/20;
Newmarkg = 0.7; Newmarkb = 1/4*(1/2+Newmarkg)^2;
dt=[0.00002*ones(1,20),0.00004*ones(1,20),0.00008*ones(1,20),0.00016*ones(1,20)];
nincr = length(dt);
maxit = 30;
    Report =@(x) (disp(x) );
utol= 1e-3;
% [fens,gcells] = block(L,W,H, 25, 2,2);
[fens,gcells] = block(L,W,H, 6, 1,3);
% [fens,gcells] = block(L,W,H, 5, 1,1);
[fens,gcells] = H8_to_H27(fens,gcells);
ir =gauss_rule (3,3);
% Material
prop = property_plastic_perf_j2_iso(struct('E',E,'nu',nu,'sigma_y',sigma_y,'rho',rho));
mater = mater_ss_plastic_perf_j2_triax(struct('property',prop));
% Finite element block
feb = feblock_defor_ss_nonlinear (struct ('mater',mater, 'gcells',gcells,...
    'integration_rule',gauss_rule(3,3)));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = 0*geom; % zero out
% Apply EBC's

ebc_fenids=fenode_select (fens,struct('box',[L,L,-Inf,Inf,-Inf,Inf],'inflate',44/10000));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=[];
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select (fens,struct('box',[0,0,-Inf,Inf,-Inf,Inf],'inflate',44/10000));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=ebc_fenids*0 +1;
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);

% Number equations
u   = numbereqns (u);

tip_fenid=fenode_select (fens,struct('box',[0,0,0,0,0,0],'inflate',44/10000));




u = u*0; % zero out the displacement
v = u*0; % zero out the Velocity
a = u*0; % zero out the Acceleration

% Initial conditions
v = u;% copy everything from displacement field to velocity
fenids=fenode_select (fens,struct('box',[0,0,-Inf,Inf,-Inf,Inf],'inflate',44/10000));
xy= get (fens,'xyz');
for j=1:length(fens)
    v = scatter(v,j,[0,0,-1]*vmag);
end
% Assemble the system matrix

ngraphics=  5;
igraphics =ngraphics-1;
if graphics
    gv=graphic_viewer;
    gv=reset (gv,struct ('limits',[0 17 -26 26]));
end
snapshot =0;
K = start (sparse_sysmat, get(u, 'neqns'));
Mems=mass(feb, geom, u);% Constant mass matrix
mat=get(Mems , 'mat');
for  j = 1:length(mat)
    mat{j}=mat{j}/(Newmarkb*dt(1)^2);
end
Mems =set(Mems,'mat',mat);
%         K = start (sparse_sysmat, get(u, 'neqns'));

Frame=0;
t=0; % time=load magnitude
incr=1;
while (incr < nincr)
    Report(['Increment ' num2str(incr) ]); % pause
    
    % Initialization
    u1 = u; % guess
    du = 0*u; % this will hold displacement increment
    u1 = apply_ebc(u1);
    du = apply_ebc(du);
    a1 = -(1/Newmarkb/dt(incr))*v -(1/2-Newmarkb)/Newmarkb*a;
    v1 = v + dt(incr)*((1-Newmarkg)*a + Newmarkg*a1);
    F0 = start (sysvec, get(u, 'neqns'));
    ap =-(1/Newmarkb/dt(incr)^2)*u -(1/Newmarkb/dt(incr))*v -((1/2-Newmarkb)/Newmarkb)*a;
    F0 = assemble (F0, inertial_force(feb, geom, (-Newmarkb*dt(incr)^2)*ap, Mems));
    iter=1;
    if graphics,
        clear(gv,[]);
        gv=reset(gv,[]);
        axis off;
        draw(feb,gv, struct ('x', geom,'u',scale*u, 'facecolor','cyan'));
        draw(feb,gv, struct ('x', geom,'u',0*u, 'facecolor','Yellow','alpha', 0.2));
        %                  print(gcf, '-r50', [mfilename n2s0p(Frame) '.png'], '-dpng');
        Frame = Frame+1;
    else
        %         plot (t,(V1'*Mmat*V1)/2,'bo'); hold on
        utip =gather(u,tip_fenid,'values');
        plot (t,utip (3),'kx'); hold on
        drawnow
    end
    while 1
        feb1=feb;
        F = F0;
        [Ignore,evs] = restoring_force(feb1,geom,u1,u);       % Internal forces
        F = assemble (F, evs);
        F = assemble (F, inertial_force(feb, geom, (-1)*u1, Mems));
        ems=cat(2,stiffness(feb1, geom, u1,u), Mems);
        K =assemble (K,ems);
%         K = external_decomposition (K,  []);
        du = scatter_sysvec(du, K\F); % Displacement increment
        %                 max(abs(get(F,'vec')))
        u1 = u1 + du;   % increment displacement
        v1 = v1 + (Newmarkg/Newmarkb/dt(incr))*du;
        a1 = a1 + (1/Newmarkb/dt(incr)^2)*du;
        K = reinitialize (K);
        if graphics,
            %                     draw(feb,gv, struct ('x', geom,'u',u1, 'facecolor','none'));
            pause(0.5); 
        end
        Report(['   It. ' num2str(iter) ': ||du||=' num2str(norm(du))]);
        if (max(abs(gather(du,[],'values'))) < utol) break; end;% convergence check
        if (iter >maxit)% bailout for failed convergence
            error( [' Possible failed convergence']);
        end
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom,u1,u);        % final update
    Report(['    Converged for t=' num2str(t)]); % pause
    u = u1;       % update the displacement
    v = v1;       % update the displacement
    a = a1;
    norm(a)
    %             utip=gather(u,cncl,'values','noreshape');
    %             tipu(end +1) = utip(1);
    %             plot (tipu); pause(0.01);
    t = t + dt(incr);
    incr = incr + 1;
    if (dt(incr)~=dt(incr-1))
        Mems=mass(feb, geom, u);% Constant mass matrix
        mat=get(Mems , 'mat');
        for  j = 1:length(mat)
            mat{j}=mat{j}/(Newmarkb*dt(1)^2);
        end
        Mems =set(Mems,'mat',mat);
    end
end
if graphics,
    gv=reset(clear(gv,[]),[]);camset (gv,cam);
    draw(feb,gv, struct ('x', geom,'u',u, 'facecolor','red'));
    draw(feb,gv, struct ('x', geom,'u',0*u, 'facecolor','Yellow'));
end