% Tetrahedral (t4) Mesh of a block located in a given range.  The
% orientation of the tetrahedra in the logical rectangular sub blocks is
% varied to produce a kind of cross-hatched pattern.
%
% function [fens,gcells] = T4_blockc(Length,Width,Height,nL,nW,nH)
%
% Range =<0,Length> x <0,Width> x <0,Height>
% Divided into elements: nL, nW, nH in the first, second, and
% third direction (x,y,z).
%
% Example: run test_t4block
%
function [fens,gcells] = T4_blockc(Length,Width,Height,nL,nW,nH)
    nnodes=(nL+1)*(nW+1)*(nH+1);
    ncells=(nL)*(nW)*(nH);
    xs=zeros(nnodes,3);
    conns=zeros(ncells,4);
    t4ia = [8, 4, 7, 5; 6, 7, 2, 5; 3, 4, 2, 7; 1, 2, 4, 5; 7, 4, 2, 5];
    t4ib = [7, 3, 6, 8; 5, 8, 6, 1; 2, 3, 1, 6; 4, 1, 3, 8; 6, 3, 1, 8];
    f=1;
    for k=1:(nH+1)
        for j=1:(nW+1)
            for i=1:(nL+1)
                xyz=[(i-1)*Length/nL (j-1)*Width/nW (k-1)*Height/nH];
                xs(f,:)=xyz;
                f=f+1;
            end
        end
    end
    fens=fenodeset(struct('id',(1:nnodes)','xyz',xs));

    gc=1;
    for i=1:nL
        for j=1:nW
            for k=1:nH
                nn=node_numbers(i,j,k,nL,nW,nH);
                if (mod (sum( [i,j,k] ),2)==0)
                    t4i =t4ib;
                else
                    t4i =t4ia;
                end
                for r=1:size(t4i,1)
                    conns(gc,:)=nn(t4i(r,:));
                    gc=gc+1;
                end
            end
        end
    end
    gcells =gcellset_T4(struct ('conn',conns));
end
function nn=node_numbers(i,j,k,nL,nW,nH)
    f=(k-1)*((nL+1)*(nW+1))+(j-1)*(nL+1)+i;
    nn=[f (f+1)  f+(nL+1)+1 f+(nL+1)];
    nn=[nn nn+((nL+1)*(nW+1))];
end

