% Import tetrahedral (4- and 10-node) mesh.
%
% function [fens,gcells]=mesh_import(filename,xyzscale,saveReadData)
%
% filename= either ABAQUS (.INP), Cosmos (.geo), or Comsol (.mphtxt) filename
% xyzscale= optional scale for the XYZ coordinates
% saveReadData= optional flag, should the data be saved in a .mat file
%     after having been read?
%
% Limitations: only the node and solid-element sections are read. Only 4-node and 10-node
% tetrahedra  are handled.
%
function [fens,gcells]=mesh_import(filename,xyzscale,saveReadData)
    if (~exist('xyzscale','var'))
        xyzscale = 1.0;
    end
    if (~exist('saveReadData','var'))
        saveReadData = ~true;
    end
    
    [pathstr, name, ext] = fileparts(filename) ;
    
    if (strcmpi(ext,'.inp'))
        [fens,gcells]=Abaqus_mesh_import(filename,xyzscale,saveReadData);
    elseif (strcmpi(ext,'.geo'))
        [fens,gcells]=geo_mesh_import(filename,xyzscale,saveReadData);
    elseif (strcmpi(ext,'.mphtxt'))
        [fens,gcells]=Comsol_mesh_import(filename,xyzscale,saveReadData);
    elseif (strcmpi(ext,'.mat'))
        [fens,gcells]=native_mesh_import(filename,xyzscale,saveReadData);
    else
        error( ['No import handler for extension ' ext]);
    end
end

