function [uSEC,uMIN,uHR,uDAY,uWK,uMONTH,uYR,uFT,uSLUG,uRAN,...
    uM,uKG,uK,uCM,uGM,uDEG,uRAD,uREV,...
    uIN,uNMI,uYD,uMM,uMILE,uG,uLB,uNT,uOZ,uLBM,uKT,...
    uMPH,uPSI,uPA,uBAR,uATM,uTORR,ummHG,uJ,...
    uCAL,uMEV,uERG,uBTU,uW,uMW,uHP,uA,uV,...
    uHZ,uRPS,uRPM,uL,uGAL,uGPM,...
    uMICRO,uKILO,uMILLI,uMEGA,uGIGA,uTERA]=physical_units(System_of_units,base_time_units)
% physical_units - a function to create unit conversion factors
%
% function [uSEC,uMIN,uHR,uDAY,uWK,uMONTH,uYR,uFT,uSLUG,uRAN,...
%     uM,uKG,uK,uCM,uGM,uDEG,uRAD,uREV,...
%     uIN,uNMI,uYD,uMM,uMILE,uG,uLB,uNT,uOZ,uLBM,uKT,...
%     uMPH,uPSI,uPA,uBAR,uATM,uTORR,ummHG,uJ,...
%     uCAL,uMEV,uERG,uBTU,uW,uMW,uHP,uA,uV,...
%     uHZ,uRPS,uRPM,uL,uGAL,uGPM,...
%     uMICRO,uKILO,uMILLI,uMEGA,uGIGA,uTERA]=physical_units(System_of_units,base_time_units)
%     
%
% Inputs:
% --System_of_units defaults to SI
% if System_of_units=='US', basic assumed units are American Engineering:
% LENGTH=uFT, TIME=uSEC, MASS=uSLUG TEMP=uRAN FORCE=uLB
% if System_of_units=='CGS', basic assumed units are Centimeter,Gram,Second:
% LENGTH=uCM, TIME=uSEC, MASS=uGM TEMP=uK FORCE=uDYNE
% otherwise, basic assumed units are SI:
% LENGTH=uM , TIME=uSEC, MASS=uKG   TEMP=uK   FORCE=uNT
% --base_time_units defaults to SEC
%
% Outputs:
%         uSEC,uMIN,uHR,uDAY,uWK,uMONTH,uYR,uFT,uSLUG,uRAN,...
%         uM,uKG,uK,uCM,uGM,uDEG,uREV,...
%         uIN,uNMI,uYD,uMM,uMILE,uG,uLB,uNT,uOZ,uLBM,uKT,...
%         uMPH,uPSI,uPA,uBAR,uATM,uTORR,ummHG,uJ,...
%         uCAL,uMEV,uERG,uBTU,uW,uMW,uHP,uA,uV,...
%         uHZ,uRPS,uRPM,uL,uGAL,uGPM
%
% uSEC = second
% % time conversions
% uMIN = Minute
% uHR = hour
% uDAY = Day
% uWK = Week
% uMONTH = Month
% uYR = Year
% uCOUL=Coulomb (charge);
% uRAD=Radian (angle)
% % prefixes
% uMICRO=10^(-6);
% uMILLI=10^(-3);
% uKILO=10^3;
% uMEGA=10^6;
% uGIGA=10^9;
% uTERA=10^12;
% uDEG=Degree (angle)
% uREV=Measure of angles in terms of revolutions
% uIN=Inch
% uNMI=Nautical mile
% uCM=Centimeter
% uYD=Yard
% uMM=Millimeter
% uMILE=Mile
% uG=Gravity acceleration
% uLB=Pound
% uNT=newton
% uOZ=ounce
% uLBM= Pound of mass
% uKT=knot
% uMPH=mile per hour
% uPSI=Pound per square inch
% uPA= Pascal
% uBAR=Bar
% uATM=Atmosphere
% uTORR=torr
% ummHG=mm Hg
% uJ= Joule
% uCAL = Calorie
% uMEV = Mega electron volt
% uERG = erg
% uBTU = BTU
% uW=watt
% uMW = Megawatt
% uHP=Horsepower
% uA = ampere
% uV = volt
% uHZ = Frequency (Hertz)
% uRPS = Revolutions per second
% uRPM =  Revolutions per minute
% uL = Liter
% uGAL = Gallon
% uGPM = Gallons per minute 
% 
% Based on UNITS/GL_UNITS by Glenn Gomes, Version 2011.

if (~exist('System_of_units'))
    System_of_units='SI';
end
if (~exist('base_time_units'))
    base_time_units='SEC';
end

if (strcmp(upper(base_time_units),'SEC'))
    uSEC = 1.0;
elseif (strcmp(upper(base_time_units),'MIN'))
    uSEC = 1/60;
elseif (strcmp(upper(base_time_units),'HR'))
    uSEC = 1/(60*60);
elseif (strcmp(upper(base_time_units),'DY'))
    uSEC = 1/(60*60*24);
elseif (strcmp(upper(base_time_units),'YR'))
    uSEC = 1/(60*60*24*365);
elseif (strcmp(upper(base_time_units),'WK'))
    uSEC = 1/(60*60*24*7);
else
    disp('The only valid entries for the time base units are: SEC|MIN|HR|DY|YR')
    return
end

% time conversions
uMIN = uSEC*60;
uHR = uSEC*60*60;
uDAY = uSEC*60*60*24;
uWK = uSEC*60*60*24*7;
uMONTH = uSEC*60*60*24*365/12;
uYR = uSEC*60*60*24*365;

% base units for charge, time, angle (all systems)
uCOUL=1.0;uRAD=1.0;

% base units for length, mass, temperature (different for each system)

if (strcmp(upper(System_of_units),'US'))
    strflag='US';
    uFT=1.0; uSLUG=1.0; uRAN=1.0;                     % US base units
    uM=uFT*3*(1/0.9144);uKG= uSLUG/14.593903591998501; uK=1.8*uRAN; % SI base units
    uCM=uM/100.0;uGM= uKG/1000.0;                     % CGS base units
elseif (strcmp(upper(System_of_units),'CGS'))
    strflag='CGS';
    uCM=1.0; uGM=1.0; uK=1.0;						  % CGS base units
    uM=100.0*uCM;uKG=1000.0*uGM; 					  % SI base units
    uFT=uM/(3*(1/0.9144));uSLUG=uKG*14.593903591998501;uRAN=uK/1.8; % US base units
else %SI is the default
    strflag='SI';
    uM=1.0; uKG=1.0; uK=1.0;						  % SI base units
    uCM=uM/100.0; uGM=uKG/1000.0;               % CGS base units
    uFT=uM/(3*(1/0.9144));uSLUG=uKG*14.593903591998501;uRAN=uK/1.8; % US base units
end

% prefixes
uMICRO=10^(-6);
uMILLI=10^(-3);
uKILO=10^3;
uMEGA=10^6;
uGIGA=10^9;
uTERA=10^12;

% angle conversions
uDEG=pi*uRAD/180.0;
uREV=2.0*pi;% Measure of angles in terms of revolutions

% length conversions
uIN=uFT/12;
uNMI=1852*uM;
uCM=uM/100.0;
uYD=3.0*uFT;
uMM=uM/1000.0;
uMILE=5280*uFT;

% acceleration conversions
uG=32.17405*uFT/uSEC^2;

% force conversions
uLB=uSLUG*uFT/uSEC^2;
uNT=uKG*uM/(uSEC^2); %newton
uOZ=uLB/16.0;

% mass conversions
uLBM=uLB/(uG);

% speed conversions
uKT=uNMI/uHR;
uMPH=uMILE/uHR;

% pressures
uPSI=uLB/(uIN^2);
uPA= uNT/(uM^2);
uBAR=10^5*uPA;
uATM=14.6959*uPSI;
uTORR=133.322*uPA;
ummHG=uTORR;

% work
uJ= uNT*uM;
uCAL = uJ * 4.1868;
uMEV = uJ / 6.242e12;
uERG = uJ * 1E-7;
uBTU = 1054*uJ;

% power
uW=uJ/uSEC;
uMW = 1e6*uW;
uHP=745.7*uW;

% electricity
uA = uCOUL/uSEC;
uV = uJ/uCOUL;

% frequency
uHZ = 1.0/uSEC;
uRPS = uHZ;% Revolutions per second
uRPM = 1/uMIN;% Revolutions per minute

% volume
uL = 1000.0*uCM^3;
uGAL = uFT^3/7.48052;

% flow rate
uGPM = uGAL/uMIN;
%
 
% Check the number of output arguments and report error if different from
% that expected.
if (nargout ~=58)
    error('Mismatched number of output arguments');
end